package converter;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;
import java.awt.image.RescaleOp;
import java.util.Vector;

public class HiresImage {

	private BufferedImage srcImage 						= null;
	private BufferedImage destImage 						= null;
	private BufferedImage destImageNotReduced 			= null;
	private BufferedImage destImageInterlaced 			= null;
	private BufferedImage destImageInterlacedLeft 		= null;
	private BufferedImage destImageInterlacedRight		= null;
	private BufferedImage destImageInterlacedNotReduced 	= null;



	private int backgroundColor = 0;


	private float brightness 	= 128f;
	private float contrast 		= 1.0f;

	private float red			= 100;
	private float green			= 100;
	private float blue			= 100;

	private float paletteValue 	= 100f;
	private int ditheringNr 		= 0;


	private int[][] baseColors = {

			{0x00,0x00,0x00},	// 0 = BLACK
			{0xff,0xff,0xff},	// 1 = WHITE
			{0xa1,0x4d,0x43},	// 2 = RED
			{0x6a,0xc1,0xc8},	// 3 = TURKIS
			{0xa2,0x57,0xa5},	// 4 = VIOLET 
			{0x5c,0xad,0x5f},	// 5 = GREEN
			{0x50,0x44,0x9d},	// 6 = BLUE
			{0xcb,0xd6,0x89},	// 7 = YELLOW
			{0xa3,0x68,0x3a},	// 8 = ORANGE
			{0x6e,0x54,0x0b},	// 9 = BROWN 
			{0xcc,0x7f,0x76},	// 10 = LT RED
			{0x63,0x63,0x63},	// 11 = DK GRAY
			{0x8b,0x8b,0x8b},	// 12 = GRAY
			{0x9c,0xe3,0x9d},	// 13 = LT GREEN
			{0x8a,0x7f,0xcd},	// 14 = LT BLUE
			{0xaf,0xaf,0xaf}		// 15 = LTGRAY
	};

	private int[][] srcColorsRGB = new int[16][3];
	private int[][] destColorsRGB = new int[16][3];


	class Block {

		int colorsUsedCount			= 0;

		int[] usedColors 			= new int[64]; // * Which colors are used in this block
		int[] usedColorsAmount 		= new int[64]; // * How many times this colors are used in this block

		int[] multiColorsIndexes 	= new int[] {-1,-1,-1};		
		byte[] data 					= new byte[8];
	}

	Block[] blocks = new Block[1000]; 


	public HiresImage() {

	}


	public void reset() {

		setBrightness(107.5f);
		setContrast(0.86f);
		setPaletteValue(110);
		setRed(100);
		setGreen(100);
		setBlue(100);
		resetPalette();
		updatePalette();
	}


	public BufferedImage getSrcImage() {		
		return srcImage;	
	}
	public BufferedImage getDestImage() {		
		return destImage;	
	}
	public BufferedImage getDestImageNotReduced() {		
		return destImageNotReduced;	
	}
	public BufferedImage getDestImageInterlaced() {		
		return destImageInterlaced;	
	}
	public BufferedImage getDestImageInterlacedLeft() {		
		return destImageInterlacedLeft;	
	}
	public BufferedImage getDestImageInterlacedRight() {		
		return destImageInterlacedRight;	
	}
	public BufferedImage getDestImageInterlacedNotReduced() {		
		return destImageInterlacedNotReduced;	
	}


	public void setBrightness(float brightness) {		
		this.brightness	= brightness;		
	}

	public void setContrast(float contrast) {		
		this.contrast = contrast;	
	}

	public void setDitheringNr(int nr) {
		this.ditheringNr = nr;
	}

	public void setRed(float value) {
		red = value;
	}

	public void setGreen(float value) {
		green = value;
	}

	public void setBlue(float value) {
		blue = value;
	}

	public void setPaletteValue(float value) {
		paletteValue = value;
		updatePalette();
	}

	public int[][] getSrcColorsRGB() {	
		return srcColorsRGB;
	}

	public int[][] getDestColorsRGB() {	
		return destColorsRGB;
	}

	public int getBackgroundColor() {
		return backgroundColor;
	}

	public int getBackgroundColorPaletteIndex() {
		return this.getPaletteIndex(backgroundColor);
	}


	public void createSrcImage(BufferedImage image) {

		// Crop it to 320x200

		BufferedImage croped =  getScaledInstance(image, 320,200, true);

		int w = 2;
		int h = 2;
		if(image.getWidth() > 320) {

			float d = (float)320/image.getWidth();
			w = (int) (image.getWidth()*d);
			h = (int) (image.getHeight()*d);
			croped = getScaledInstance(image, w,h, true);
		}

		if(croped.getHeight() > 200) {

			float d = (float)200/image.getHeight();
			w = (int) (image.getWidth()*d);
			h = (int) (image.getHeight()*d);
			croped = getScaledInstance(image, w,h, true);
		}

		int diffX = 0;
		if(croped.getWidth() < 320) diffX = (320-croped.getWidth())/2;

		int diffY = 0;
		if(croped.getHeight() < 200) diffY = (200-croped.getHeight())/2;


		srcImage = new BufferedImage(320, 200, BufferedImage.TYPE_INT_RGB);

		Graphics g = srcImage.createGraphics();
		g.drawImage(croped, diffX,diffY, null);
	}


	public void updateDestImages(boolean interlaced) {
		if(srcImage == null) return;

		try {

			BufferedImage tempImage = getScaledInstance(srcImage, 320,200, false);

			// * Do new values with the Palette Luminanz slider *
			for(int y=0;y<200;y++) {
				for(int x=0;x<320;x++) {

					int color = tempImage.getRGB(x, y) & 0x00ffffff;			
					int r = (int) (((color >> 16) & 0xff) * (red/100)); 
					int g = (int) (((color >> 8) & 0xff) * (green/100)); 
					int b = (int) ((color & 0xff) * (blue/100)); 	
					tempImage.setRGB(x, y, r << 16 | g << 8 | b);
				}
			}

			// * Do Brightness and Contrast *
			RescaleOp rescaleOp2 = new RescaleOp(contrast, -128 + brightness, null); 
			rescaleOp2.filter(tempImage, tempImage);  

			// * Do Dithering *
			if(ditheringNr == 1) tempImage = order2x2(tempImage);			
			if(ditheringNr == 2) tempImage = order3x3(tempImage);			
			if(ditheringNr == 3) tempImage = order4x4(tempImage);			
			if(ditheringNr == 4) tempImage = order8x8(tempImage);			



			// * Reduce Colors *
			int block = 0;

			// Do this only if selected, because this will reduce the destImage Colors!
			if(interlaced) {

				destImageInterlaced = getScaledInstance(tempImage, 320,200, false);

				// * Reduce the Colors to 16 *
				for(int y=0;y<200;y++) {
					for(int x=0;x<320;x++) {

						int color = destImageInterlaced.getRGB(x, y) & 0x00ffffff;							
						destImageInterlaced.setRGB(x, y, getNearestPaletteColor(color));
					}							
				}		
				destImageInterlacedNotReduced = getScaledInstance(destImageInterlaced, 320,200, false);


				resetBlocks();

				backgroundColor = calculateBackgroundColor(destImageInterlaced, interlaced);

				for(int y=0;y<200;y+=8) {
					for(int x=0;x<320;x+=8) {

						for(int yy=0;yy<8;yy++) {
							for(int xx=0;xx<8;xx++) {

								int color = destImageInterlaced.getRGB(x+xx, y+yy) & 0x00ffffff;	
								addBlockColor(block, color);
							}
						}	

						reduceColorBlock(block, backgroundColor, interlaced);

						block++;
					}							
				}		

				destImageInterlacedLeft 		= makeInterlaced(destImageInterlaced, true);
				destImageInterlacedRight 	= makeInterlaced(destImageInterlaced, false);

			}
			else {

				destImage = getScaledInstance(tempImage, 320,200, false);

				// * Reduce the Colors to 16 *
				for(int y=0;y<200;y++) {
					for(int x=0;x<320;x+=2) {

						int color = destImage.getRGB(x, y) & 0x00ffffff;							
						destImage.setRGB(x, y, getNearestPaletteColor(color));
						destImage.setRGB(x+1, y, getNearestPaletteColor(color));
					}							
				}		
				destImageNotReduced = getScaledInstance(destImage, 320,200, false);


				resetBlocks();
				block = 0;

				backgroundColor = calculateBackgroundColor(destImage, interlaced);


				for(int y=0;y<200;y+=8) {
					for(int x=0;x<320;x+=8) {

						for(int yy=0;yy<8;yy++) {
							for(int xx=0;xx<8;xx+=2) {

								int color = destImage.getRGB(x+xx, y+yy) & 0x00ffffff;	
								addBlockColor(block, color);
							}
						}	

						reduceColorBlock(block, backgroundColor, interlaced);

						block++;
					}							
				}		
			}			
		} 
		catch (Exception e) {
			e.printStackTrace();
		}
	}


	public byte[] convertToRawData(boolean interlaced) {

		if(srcImage == null) return null;

		int size = 0x2000+0x400+0x400;
		if(interlaced) size = 0x2000+0x2000+0x400+0x400;			
		byte[] rawData = new byte[size];

		try {

			int index = 0;
			for(int block=0;block<1000;block++) {

				int[] pos = getBlockRasterPos(block);

				for(int y=0;y<8;y++) {


					if(!interlaced) {

						int b = 0;
						for(int x=0;x<8;x+=2) {

							int 	c = destImage.getRGB(pos[0]+x, pos[1]+y) & 0x00ffffff;

							if(blocks[block].multiColorsIndexes[0] > -1 && c == getColorFromPaletteIndex(blocks[block].multiColorsIndexes[0]))			b |= (1 << (6-x));
							else if(blocks[block].multiColorsIndexes[1] > -1 && c == getColorFromPaletteIndex(blocks[block].multiColorsIndexes[1])) 	b |= (2 << (6-x));
							else if(blocks[block].multiColorsIndexes[2] > -1 && c == getColorFromPaletteIndex(blocks[block].multiColorsIndexes[2])) 	b |= (3 << (6-x));
							else b |= (0 << (6-x));		
						}	

						rawData[index] = (byte)b;
						index++;
					}
					else {

						int bLeft = 0;
						for(int x=0;x<8;x+=2) {

							int 	c = destImageInterlaced.getRGB(pos[0]+x, pos[1]+y) & 0x00ffffff;

							if(blocks[block].multiColorsIndexes[0] > -1 && c == getColorFromPaletteIndex(blocks[block].multiColorsIndexes[0]))			bLeft |= (1 << (6-x));
							else if(blocks[block].multiColorsIndexes[1] > -1 && c == getColorFromPaletteIndex(blocks[block].multiColorsIndexes[1])) 	bLeft |= (2 << (6-x));
							else if(blocks[block].multiColorsIndexes[2] > -1 && c == getColorFromPaletteIndex(blocks[block].multiColorsIndexes[2])) 	bLeft |= (3 << (6-x));
							else bLeft |= (0 << (6-x));		
						}

						int bRight = 0;
						for(int x=0;x<8;x+=2) {

							int 	c = destImageInterlaced.getRGB(pos[0]+x+1, pos[1]+y) & 0x00ffffff;

							if(blocks[block].multiColorsIndexes[0] > -1 && c == getColorFromPaletteIndex(blocks[block].multiColorsIndexes[0]))			bRight |= (1 << (6-x));
							else if(blocks[block].multiColorsIndexes[1] > -1 && c == getColorFromPaletteIndex(blocks[block].multiColorsIndexes[1])) 	bRight |= (2 << (6-x));
							else if(blocks[block].multiColorsIndexes[2] > -1 && c == getColorFromPaletteIndex(blocks[block].multiColorsIndexes[2])) 	bRight |= (3 << (6-x));
							else bRight |= (0 << (6-x));		
						}		

						rawData[index] = (byte)bLeft;
						rawData[0x2000+index] = (byte)bRight;
						index++;
					}					
				}

				if(!interlaced) {

					// * Screen Ram (0400): Colors #1 und #2 *
					rawData[0x2000 + block] = (byte)((blocks[block].multiColorsIndexes[0] & 0xF) << 4 |  (blocks[block].multiColorsIndexes[1] & 0xF));

					// * Color Ram (D800): Color #3 *
					rawData[0x2000 + 0x400 + block] =  (byte)(blocks[block].multiColorsIndexes[2] & 0xF);

				}
				else {

					// * Screen Ram (0400): Colors #1 und #2 *
					rawData[0x2000 + 0x2000 + block] = (byte)((blocks[block].multiColorsIndexes[0] & 0xF) << 4 |  (blocks[block].multiColorsIndexes[1] & 0xF));

					// * Color Ram (D800): Color #3 *
					rawData[0x2000 + 0x2000 + 0x400 + block] =  (byte)(blocks[block].multiColorsIndexes[2] & 0xF);

				}
			}

		} catch (Exception e) {
			e.printStackTrace();
		}

		return rawData;
	}


	private int calculateBackgroundColor(BufferedImage image, boolean interlaced) {

		Vector<Integer> colors = new Vector<Integer>();
		Vector<Integer> amount = new Vector<Integer>();

		int steps = 2;
		if(interlaced) steps = 1;

		for(int y=0;y<200;y++) {
			for(int x=0;x<320;x+=steps) {

				int color = image.getRGB(x, y) & 0x00ffffff;	

				boolean exist = false;
				for(int i=0;i<colors.size();i++) {

					if(colors.elementAt(i) == color) {								
						amount.set(i, amount.elementAt(i) + 1);
						exist = true;
						break;
					}
				}
				if(!exist) {

					colors.add(color);
					amount.add(1);
				}
			}							
		}	

		int highestAmount = 0;
		int mostUsedColor = 0;
		for(int i=0;i<colors.size();i++) {

			if( amount.elementAt(i) > highestAmount) {
				highestAmount = amount.elementAt(i);			
				mostUsedColor = colors.elementAt(i);

			}			
		}		
		return mostUsedColor;
	}


	private void resetBlocks() {

		for(int i=0;i<1000;i++) {

			blocks[i] = null;
			blocks[i] = new Block();
		}
	}

	private boolean addBlockColor(int block, int c) {

		for(int i=0;i<blocks[block].colorsUsedCount;i++) {
			if(c == blocks[block].usedColors[i]) {

				blocks[block].usedColorsAmount[i]++;
				return true;
			}
		}

		blocks[block].usedColors[blocks[block].colorsUsedCount] 			= c;
		blocks[block].usedColorsAmount[blocks[block].colorsUsedCount] 	= 1;
		blocks[block].colorsUsedCount++;

		return false;
	}

	private void reduceColorBlock(int block, int bgColor, boolean interlaced) {

		// * Sort all found Colors of the Blocks (8x8), so the less used colors are up *
		sortAmount(block);

		// * Replace the less used Colors with the 4 most used colors *
		reduceColors(block, bgColor, interlaced);
	}

	private void sortAmount(int block) {

		int tempColor = 0;
		int tempAmount = 0;

		int usedColorsAmount = blocks[block].colorsUsedCount;
		for(int i=0;i<usedColorsAmount-1;i++) {
			for(int ii=i+1;ii<usedColorsAmount;ii++) {

				if(blocks[block].usedColorsAmount[ii] < blocks[block].usedColorsAmount[i]) {

					tempColor 							= blocks[block].usedColors[i];
					tempAmount 							= blocks[block].usedColorsAmount[i];
					blocks[block].usedColors[i] 			= blocks[block].usedColors[ii];
					blocks[block].usedColorsAmount[i] 	= blocks[block].usedColorsAmount[ii];
					blocks[block].usedColors[ii] 		= tempColor;
					blocks[block].usedColorsAmount[ii] 	= tempAmount;					
				}
			}
		}		
	}

	private void reduceColors(int block, int bgColor, boolean interlaced) {

		if(blocks[block].colorsUsedCount <= 3) {

			// * Fill the Index Colors *
			int cIndex = 0;
			for(int i=0;i<blocks[block].colorsUsedCount;i++) {

				// * Ignore the Background Color! *
				if(blocks[block].usedColors[i] != bgColor) {

					blocks[block].multiColorsIndexes[cIndex] = getPaletteIndex(blocks[block].usedColors[i]);
					cIndex++;
				}
			}

			return;
		}


		int usedColorsAmount 	= blocks[block].colorsUsedCount;
		int colorsStartIndex 	= blocks[block].colorsUsedCount-4;


		// * If the Block has no background Color, then reduce the Colors to 3 Colors! *				
		boolean foundBackgroundColor = false;
		for(int i=colorsStartIndex;i<usedColorsAmount;i++) {

			if(blocks[block].usedColors[i] == bgColor) {
				foundBackgroundColor = true;
				break;
			}
		}
		if(!foundBackgroundColor) colorsStartIndex 	= blocks[block].colorsUsedCount-3;


		// * Fill the Index Colors *
		int cIndex = 0;
		for(int i=colorsStartIndex;i<usedColorsAmount;i++) {

			// * Ignore the Background Color! *
			if(blocks[block].usedColors[i] != bgColor) {

				blocks[block].multiColorsIndexes[cIndex] = getPaletteIndex(blocks[block].usedColors[i]);
				cIndex++;
			}
		}	


		// * Got Through all too much Colors *
		for(int i=0;i<colorsStartIndex;i++) {

			int selectedColor = -1;
			int lowestColorDiffValue = 17000000;

			// * Go through the most 4 used Colors (last Colors at the end of this Array = Base Colors) *
			// * Find the nearest Color to remove the > 4 Color
			for(int ii=colorsStartIndex;ii<usedColorsAmount;ii++) {

				int colorDiff  = Math.abs(blocks[block].usedColors[i]-blocks[block].usedColors[ii]);					
				if(colorDiff < lowestColorDiffValue) {

					selectedColor = blocks[block].usedColors[ii]; // use this color
					lowestColorDiffValue = colorDiff;		
				}
			}

			// * Override the  > 4 Color (selected Color) with one of the 4 Base Colors (nearest) * 
			if(lowestColorDiffValue != 17000000) {

				int[] blockRasterPos = getBlockRasterPos(block);
				int x = blockRasterPos[0];
				int y = blockRasterPos[1];				


				if(!interlaced) {

					for(int yy=0;yy<8;yy++) {
						for(int xx=0;xx<8;xx+=2) {

							if((destImage.getRGB(x+xx, y+yy) & 0x00ffffff) == blocks[block].usedColors[i]) {							
								destImage.setRGB(x+xx, y+yy, selectedColor);
								destImage.setRGB(x+xx+1, y+yy, selectedColor);
							}
						}
					}	
				}
				else {

					for(int yy=0;yy<8;yy++) {
						for(int xx=0;xx<8;xx++) {

							if((destImageInterlaced.getRGB(x+xx, y+yy) & 0x00ffffff) == blocks[block].usedColors[i]) {							
								destImageInterlaced.setRGB(x+xx, y+yy, selectedColor);
							}
						}
					}	
				}
			}
		}		
	}


	private void resetPalette() {

		for(int i=0;i<16;i++) {

			srcColorsRGB[i][0] = baseColors[i][0];
			srcColorsRGB[i][1] = baseColors[i][1];
			srcColorsRGB[i][2] = baseColors[i][2];	
		}
	}

	public void updatePalette() {

		for(int i=0;i<16;i++) {

			int r = (int) (srcColorsRGB[i][0] * (paletteValue/100));
			int g = (int) (srcColorsRGB[i][1] * (paletteValue/100));
			int b = (int) (srcColorsRGB[i][2] * (paletteValue/100));

			if(r > 255) r = 255;
			if(g > 255) g = 255;
			if(b > 255) b = 255;

			destColorsRGB[i][0] = r;
			destColorsRGB[i][1] = g;
			destColorsRGB[i][2] = b;			
		}
	}


	private int getNearestPaletteColor(int color) {

		int colorNr 	= 0;

		int R = (color >> 16) & 0xff;
		int G = (color >> 8) & 0xff;
		int B = color & 0xff;

		double absSum = 25555;	
		for(int i=0;i<16;i++) {

			double abs = (R - destColorsRGB[i][0]) * (R - destColorsRGB[i][0]) + (G - destColorsRGB[i][1]) * (G - destColorsRGB[i][1]) + (B - destColorsRGB[i][2]) * (B - destColorsRGB[i][2]);
			abs = Math.sqrt(abs);

			if (abs < absSum) {

				absSum = abs;
				colorNr = i;
			}
		}		

		return (destColorsRGB[colorNr][0] << 16 | destColorsRGB[colorNr][1] << 8 | destColorsRGB[colorNr][2]);
	}


	private int getPaletteIndex(int color) {

		int R = (color >> 16) & 0xff;
		int G = (color >> 8) & 0xff;
		int B = color & 0xff;

		for(int i=0;i<16;i++) {
			if(destColorsRGB[i][0] == R && destColorsRGB[i][1] == G &&destColorsRGB[i][2] == B) return i;
		}		

		return 0;
	}

	private int getColorFromPaletteIndex(int index) {
		return (destColorsRGB[index][0] << 16 | destColorsRGB[index][1] << 8 | destColorsRGB[index][2]);
	}


	private int[] getBlockRasterPos(int block) {

		int x = (int)(block*8) - (int)(block/40)*320;
		int y = block/40*8;

		return new int[] {x,y};
	}


	private int getBlock(int x, int y) {		
		return  y/8*40 + x/8;
	}


	static public BufferedImage getScaledInstance(BufferedImage img, int targetWidth,	int targetHeight, boolean higherQuality) {

		BufferedImage ret = (BufferedImage)img;
		int w, h;

		if (higherQuality) {

			w = img.getWidth();
			h = img.getHeight();
		} 
		else {
			w = targetWidth;
			h = targetHeight;
		}

		do {

			// * Check div with 2, if not fix it! *
			if (targetWidth % 2 != 0) targetWidth +=1;
			if (targetHeight % 2 != 0) targetHeight +=1;


			if (higherQuality && w > targetWidth) {
				w /= 2;
				if (w < targetWidth) {
					w = targetWidth;
				}
			}
			else w = targetWidth; // * Stop to reduce the Image size *


			if (higherQuality && h > targetHeight) {
				h /= 2;
				if (h < targetHeight) {
					h = targetHeight;
				}
			}
			else h = targetHeight;  // * Stop to reduce the Image size *

			BufferedImage tmp = new BufferedImage(w, h, BufferedImage.TYPE_INT_RGB);
			Graphics2D g2 = tmp.createGraphics();
			if(higherQuality) {
				g2.setRenderingHint(RenderingHints.KEY_ALPHA_INTERPOLATION, RenderingHints.VALUE_ALPHA_INTERPOLATION_QUALITY);
				g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);				
				g2.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BICUBIC);
			}
			g2.drawImage(ret, 0, 0, w, h, null);
			g2.dispose();

			ret = tmp;
		} 
		while (w != targetWidth || h != targetHeight);

		return ret;
	}

	private BufferedImage makeInterlaced(BufferedImage image, boolean leftSide) {

		BufferedImage tmp = getScaledInstance(image, 320,200, false);

		for(int y=0;y<200;y++) {
			for(int x=0;x<320;x+=2) {

				if(leftSide) 	tmp.setRGB(x+1, y, tmp.getRGB(x, y));
				else			tmp.setRGB(x, y, tmp.getRGB(x+1, y));
			}
		}			
		return tmp;
	}	


	public static BufferedImage order2x2(BufferedImage image) {
		int matrix[][] = {{ 1, 3 },{ 4, 2 },};

		BufferedImage newImage = new BufferedImage(image.getWidth(), image.getHeight(), BufferedImage.TYPE_INT_RGB);
		
		int width 	= image.getWidth();
		int height 	= image.getHeight();
		
		for (int y = 0; y < height; y++) {
			for (int x = 0; x < width; x++) {

				Color col = new Color(image.getRGB(x, y), true);				
				int r 	= col.getRed();
				int g 	= col.getGreen();
				int b 	= col.getBlue();
				r 		= clampValue(r + (r * matrix[x%2][y%2])/5);
				g 		= clampValue(g + (g * matrix[x%2][y%2])/5);
				b 		= clampValue(b + (b * matrix[x%2][y%2])/5);
				newImage.setRGB(x, y, new Color(r, g, b).getRGB());
			}
		}

		return newImage;
	}



	public static BufferedImage order3x3(BufferedImage image) {
		int matrix[][] = {{ 3, 7, 4},{ 6, 1, 9 },{ 2, 8, 5 }};

		BufferedImage newImage = new BufferedImage(image.getWidth(), image.getHeight(), BufferedImage.TYPE_INT_RGB);

		int width 	= image.getWidth();
		int height 	= image.getHeight();
		
		for (int y = 0; y < height; y++) {
			for (int x = 0; x < width; x++) {

				Color col = new Color(image.getRGB(x, y), true);
				int r 	= col.getRed();
				int g 	= col.getGreen();
				int b 	= col.getBlue();
				r 		= clampValue(r + (r * matrix[x%3][y%3])/10);
				g 		= clampValue(g + (g * matrix[x%3][y%3])/10);
				b 		= clampValue(b + (b * matrix[x%3][y%3])/10);
				newImage.setRGB(x, y, new Color(r, g, b).getRGB());
			}
		}

		return newImage;
	}

	public static BufferedImage order4x4(BufferedImage image) {
		int matrix[][] = { { 1, 9, 3, 11 },{ 13, 5, 15, 7 },{ 4, 12, 2, 10 },{ 16, 8, 14, 6 } };

		BufferedImage newImage = new BufferedImage(image.getWidth(), image.getHeight(), BufferedImage.TYPE_INT_RGB);
	
		int width 	= image.getWidth();
		int height 	= image.getHeight();
		
		for (int y = 0; y < height; y++) {
			for (int x = 0; x < width; x++) {

				Color col = new Color(image.getRGB(x, y), true);
				int r 	= col.getRed();
				int g 	= col.getGreen();
				int b 	= col.getBlue();
				r 		= clampValue(r + (r* matrix[x%4][y%4])/17);
				g 		= clampValue(g + (g* matrix[x%4][y%4])/17);
				b 		= clampValue(b + (b* matrix[x%4][y%4])/17);
				newImage.setRGB(x, y, new Color(r, g, b).getRGB());
			}
		}

		return newImage;
	}

	public static BufferedImage order8x8(BufferedImage image) {
		int matrix[][] = {{ 1 ,49 ,13 ,61 ,4 ,52 ,16 ,64 },{ 33 ,17 ,45 ,29 ,36 ,20 ,48 ,32 },{ 9 ,57 ,5 ,53 ,12 ,60 ,8 ,56 },{ 41 ,25 ,37 ,21 ,44 ,28 ,40 ,24},{3 ,51 ,15 ,63 ,2 ,50 ,14 ,62},{35 ,19 ,47 ,31 ,34 ,18 ,46 ,30},{11 ,59 ,7 ,55 ,10 ,58 ,6 ,54},{43 ,27 ,39 ,23 ,42 ,26 ,38 ,22}};

		BufferedImage newImage = new BufferedImage(image.getWidth(), image.getHeight(), BufferedImage.TYPE_INT_RGB);

		int width 	= image.getWidth();
		int height 	= image.getHeight();
		
		for (int y = 0; y < height; y++) {
			for (int x = 0; x < width; x++) {

				Color col = new Color(image.getRGB(x, y), true);
				int r 	= col.getRed();
				int g 	= col.getGreen();
				int b 	= col.getBlue();
				r 		= clampValue(r + (r*matrix[x%8][y%8])/65);
				g 		= clampValue(g + (g*matrix[x%8][y%8])/65);
				b 		= clampValue(b + (b*matrix[x%8][y%8])/65);
				newImage.setRGB(x, y, new Color(r, g, b).getRGB());
			}
		}
		return newImage;
	}

	private static int clampValue(int val) {		
		return (int)(val < 0 ? 0 : val > 255 ? 255 : val);
	}
}
