package converter.tools;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GraphicsEnvironment;
import java.awt.RenderingHints;
import java.awt.color.ColorSpace;
import java.awt.color.ICC_ColorSpace;
import java.awt.color.ICC_Profile;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferedImage;
import java.awt.image.ColorConvertOp;
import java.io.IOException;
import java.io.InputStream;
import java.util.prefs.Preferences;

import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;

import ffilechooser.FButton;
import ffilechooser.FFrame;
import ffilechooser.FLabel;
import ffilechooser.FRadioButton;
import ffilechooser.FTextField;

public class ColorPicker extends JDialog {

	JDialog colorPicker;


	static Preferences prefs = Preferences.userNodeForPackage(ColorPicker.class);

	final static String PREF_NAME_colorpicker_mode		= "colorpicker_mode";
	final static String PREF_NAME_foreground_color		= "foreground_color";
	final static String PREF_NAME_background_color		= "background_color";	

	private int colorPickerMode = 0;
	private Color foregroundColor = Color.WHITE;
	private Color backgroundColor = Color.BLACK;


	int width;
	int height;

	Color newColor 		= Color.GREEN;
	Color currentColor 	= Color.BLACK;

	ICC_Profile iccCMYKProfile;
	ICC_Profile iccRGBProfile;	
	ColorSpace cmykColorspace;
	ColorSpace rgbColorspace;


	JLabel label;

	JComponent colorPanel;
	JComponent colorVerticalPanel;
	JComponent colorsNewCurrent;

	int colorPanelXPos;
	int colorPanelYPos;
	int colorVerticalPanelXPos;
	int colorVerticalPanelYPos;
	int colorVerticalPanelWidth;
	int colorVerticalPanelHeight;
	int verticalImageWidth;
	int verticalImageHeight;

	int colorArrowSize = 0;
	int colorArrowYPos;

	int colorsNewCurrentXPos;
	int colorsNewCurrentYPos;		
	int colorsNewCurrentWidth;
	int colorsNewCurrentHeight; 


	BufferedImage saturationImage;
	BufferedImage brightnessImage;

	BufferedImage colorImage;
	BufferedImage verticalColorImage;


	static public int COLOR_IMAGE_TYPE_CIRCLE 			= 0;
	static public int COLOR_IMAGE_TYPE_HUE_BOX 			= 1;
	static public int COLOR_IMAGE_TYPE_SATURATION_BOX 	= 2;
	static public int COLOR_IMAGE_TYPE_BRIGHTNESS_BOX 	= 3;
	static public int COLOR_IMAGE_TYPE_RED_BOX 			= 4;
	static public int COLOR_IMAGE_TYPE_GREEN_BOX 		= 5;
	static public int COLOR_IMAGE_TYPE_BLUE_BOX 			= 6;

	private int colorImageType = COLOR_IMAGE_TYPE_HUE_BOX;

	private float hue 			= 359;
	private float saturation 	= 1.0f;
	private float brightness 	= 1.0f;

	int colorBoxClicked = 1; // 1 = foreground, 2 = background

	Font baseFont;



	int colorRingXPos = 0;
	int colorRingYPos = 0;

	boolean init = true;

	private FLabel hueLabel;
	private FLabel satLabel;
	private FLabel briLabel;
	private FLabel redLabel;
	private FLabel greenLabel;
	private FLabel blueLabel;
	private FLabel cLabel;
	private FLabel mLabel;
	private FLabel yLabel;
	private FLabel kLabel;

	private FTextField hueTextField;
	private FTextField satTextField;
	private FTextField briTextField;
	private FTextField redTextField;
	private FTextField greenTextField;
	private FTextField blueTextField;
	private FTextField cTextField;
	private FTextField mTextField;
	private FTextField yTextField;
	private FTextField kTextField;

	private FRadioButton hueRadioButton;
	private FRadioButton satRadioButton;
	private FRadioButton briRadioButton;
	private FRadioButton redRadioButton;
	private FRadioButton greenRadioButton;
	private FRadioButton blueRadioButton;

	private FFrame fframe;


	public interface ClickListener {
		void okClicked();
	}

	private ClickListener clickListener;

	public ColorPicker(final FFrame fframe, ClickListener clickListener, Color initColor) {

		try {

			InputStream in = ClassLoader.class.getResourceAsStream("/icc/cmyk.icc");
			iccCMYKProfile = ICC_Profile.getInstance(in);
			cmykColorspace = new ICC_ColorSpace(iccCMYKProfile);

			in = ClassLoader.class.getResourceAsStream("/icc/rgb.icc");
			iccRGBProfile = ICC_Profile.getInstance(in);
			rgbColorspace = new ICC_ColorSpace(iccRGBProfile);

		} 
		catch (Exception e) {			
			e.printStackTrace();
		} 


		try {

			InputStream in = ClassLoader.class.getResourceAsStream("/font/font.ttf");
			Font baseFont = Font.createFont(Font.TRUETYPE_FONT, in);
			GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
			ge.registerFont(baseFont);	

			in = ClassLoader.class.getResourceAsStream("/font/font_light.ttf");
			baseFont = Font.createFont(Font.TRUETYPE_FONT, in);
			ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
			ge.registerFont(baseFont);	
		} 
		catch (Exception e) {			
			e.printStackTrace();
		} 



		colorPicker 				= this;
		this.fframe 				= fframe;
		this.clickListener 		= clickListener;

		colorPickerMode = prefs.getInt(PREF_NAME_colorpicker_mode, 0);	

		if(initColor == null) {
			String foregroundColorString = prefs.get(PREF_NAME_foreground_color, "#ffffff");		
			String backgroundColorString = prefs.get(PREF_NAME_background_color, "#000000");		

			foregroundColor = Color.decode(foregroundColorString);
			backgroundColor = Color.decode(backgroundColorString);
		}

		int width 	= fframe.monitor.convertFloatToRealWidth(0.14f);
		int height 	= fframe.monitor.convertFloatToRealHeight(0.25f);

		saturationImage 	= createSaturationColorBoxImage(width-1,height-1);
		brightnessImage 	= createBrightnessColorBoxImage(width-1,height-1);


		addWindowListener(new WindowAdapter()	{
			public void windowClosing(WindowEvent e) {
				dispose();		
			}
		});


		colorPanel = new JComponent() {

			@Override
			public void paintComponent(Graphics g) {
				super.paintComponent(g);


				Graphics2D g2 = (Graphics2D)g;
				g2.setRenderingHint(RenderingHints.KEY_ALPHA_INTERPOLATION, RenderingHints.VALUE_ALPHA_INTERPOLATION_QUALITY);
				g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
				g2.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);

				g2.drawImage(colorImage, 0, 0, colorImage.getWidth(), colorImage.getHeight(), null);	

				// * Draw Black Frame Circle or Rectangle *
				g2.setColor(new Color(50,50,50));
				if(colorImageType == COLOR_IMAGE_TYPE_CIRCLE)	g2.drawOval(0, 0, colorImage.getWidth()-2, colorImage.getHeight()-1);	
				else 											g2.drawRect(0, 0, colorImage.getWidth()-1, colorImage.getHeight()-1);	


				int size = fframe.monitor.convertFloatToRealHeight(0.01f);

				g2.setColor(Color.BLACK);
				g2.drawOval(colorRingXPos-size/2, colorRingYPos-size/2, size,size);	
				g2.setColor(Color.WHITE);
				g2.drawOval(colorRingXPos-1-size/2, colorRingYPos-1-size/2, size+2,size+2);	

				g2.dispose();
			}			
		};
		colorPanel.addMouseListener(new MouseListener() {

			@Override
			public void mouseClicked(MouseEvent e) {
			}

			@Override
			public void mousePressed(MouseEvent e) {
				mouseEventColorPanel(e);
			}
			@Override
			public void mouseReleased(MouseEvent e) {				
			}
			@Override
			public void mouseEntered(MouseEvent e) {				
			}
			@Override
			public void mouseExited(MouseEvent e) {				
			}			
		});
		colorPanel.addMouseMotionListener(new MouseMotionListener() {

			@Override
			public void mouseDragged(MouseEvent e) {
				mouseEventColorPanel(e);			
			}

			@Override
			public void mouseMoved(MouseEvent e) {
			}			
		});
		add(colorPanel);


		colorVerticalPanel = new JComponent() {

			@Override
			public void paintComponent(Graphics g) {
				super.paintComponent(g);

				Graphics2D g2 = (Graphics2D)g;
				g2.setRenderingHint(RenderingHints.KEY_ALPHA_INTERPOLATION, RenderingHints.VALUE_ALPHA_INTERPOLATION_QUALITY);
				g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
				g2.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);

				g2.drawImage(verticalColorImage, (int)((float)(colorVerticalPanelWidth-verticalImageWidth)/2.0f), colorArrowSize, verticalImageWidth, verticalImageHeight, null);	

				g2.setColor(new Color(50,50,50));
				g2.drawRect((int)((float)(colorVerticalPanelWidth-verticalImageWidth)/2.0f), colorArrowSize, verticalImageWidth-1, verticalImageHeight-1);	


				// * Arrows *
				g2.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_NEAREST_NEIGHBOR);	
				g2.setColor(Color.BLACK);	

				int xPos 	= (int)((float)(colorVerticalPanelWidth-verticalImageWidth)/2.0f)-1;	
				g2.drawPolygon(new int[] {xPos-colorArrowSize, xPos-colorArrowSize, xPos}, new int[] {colorArrowYPos-colorArrowSize, colorArrowYPos+colorArrowSize, colorArrowYPos}, 3);

				xPos 	= (int)((float)(colorVerticalPanelWidth-verticalImageWidth)/2.0f) + verticalColorImage.getWidth();	
				g2.drawPolygon(new int[] {xPos+colorArrowSize, xPos+colorArrowSize, xPos}, new int[] {colorArrowYPos-colorArrowSize, colorArrowYPos+colorArrowSize, colorArrowYPos}, 3);

				g2.dispose();
			}			
		};

		colorVerticalPanel.addMouseListener(new MouseListener() {

			@Override
			public void mouseClicked(MouseEvent e) {		
				mouseEventColorVerticalPanel(e);
			}

			@Override
			public void mousePressed(MouseEvent e) {
			}
			@Override
			public void mouseReleased(MouseEvent e) {
			}
			@Override
			public void mouseEntered(MouseEvent e) {
			}
			@Override
			public void mouseExited(MouseEvent e) {				
			}		
		});


		colorVerticalPanel.addMouseMotionListener(new MouseMotionListener() {

			@Override
			public void mouseDragged(MouseEvent e) {
				mouseEventColorVerticalPanel(e);
			}

			@Override
			public void mouseMoved(MouseEvent e) {
			}			
		});
		add(colorVerticalPanel);


		colorsNewCurrent = new JComponent() {

			@Override
			public void paintComponent(Graphics g) {
				super.paintComponent(g);

				Graphics2D g2 = (Graphics2D)g;
				g2.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_NEAREST_NEIGHBOR);

				g2.setColor(newColor);				
				g2.fillRect(0, 0, getWidth()-1, getHeight()/2);	

				g2.setColor(currentColor);
				g2.fillRect(0, getHeight()/2, getWidth()-1, getHeight()/2);	

				g2.setColor(new Color(50,50,50));				
				g2.drawRect(0, 0, getWidth()-1, getHeight()-1);	


				g2.dispose();
			}			
		};
		add(colorsNewCurrent);


		updateFont();

		FButton okButton = new FButton(fframe);
		okButton.setBounds(0.26f, 0.034f, 0.032f, 0.022f);
		okButton.setBorderColor(Color.BLACK);
		okButton.setBorderRadius(0.007f);
		okButton.setFont(baseFont);
		okButton.setText("OK");
		okButton.addMouseListener(new MouseListener() {
			@Override
			public void mouseClicked(MouseEvent e) {}
			@Override
			public void mousePressed(MouseEvent e) {

				if(colorBoxClicked == 1) 	setForegroundColor(newColor);
				else							setBackgroundColor(newColor);	

				clickListener.okClicked();
				dispose();
			}
			@Override
			public void mouseReleased(MouseEvent e) {}
			@Override
			public void mouseEntered(MouseEvent e) {}
			@Override
			public void mouseExited(MouseEvent e) {}			
		});
		add(okButton);	

		FButton cancelButton = new FButton(fframe);
		cancelButton.setBounds(0.26f, 0.062f, 0.032f, 0.022f);
		cancelButton.setBorderColor(Color.BLACK);
		cancelButton.setBorderRadius(0.007f);
		cancelButton.setFont(baseFont);
		cancelButton.setText("Cancel");
		cancelButton.addMouseListener(new MouseListener() {
			@Override
			public void mouseClicked(MouseEvent e) {}
			@Override
			public void mousePressed(MouseEvent e) {
				dispose();				
			}
			@Override
			public void mouseReleased(MouseEvent e) {}
			@Override
			public void mouseEntered(MouseEvent e) {}
			@Override
			public void mouseExited(MouseEvent e) {}			
		});
		add(cancelButton);	



		KeyListener textFieldKeyListener = new KeyListener() {

			@Override
			public void keyTyped(KeyEvent e) {

				char c = e.getKeyChar();

				int len = ((FTextField)e.getSource()).getText().length();					
				if(len >= 4) {
					e.consume();
					return;				
				}

				if (!((c >= '0') && (c <= '9') || (c == KeyEvent.VK_BACK_SPACE) || (c == KeyEvent.VK_DELETE))) e.consume();				
			}

			@Override
			public void keyPressed(KeyEvent e) {

				if(e.getKeyCode() == 10) {

					if(e.getSource().equals(hueTextField) || e.getSource().equals(satTextField) || e.getSource().equals(briTextField)) {

						hue 			= (float)Integer.parseInt(hueTextField.getText().trim())/360;
						saturation 	= (float)Integer.parseInt(satTextField.getText().trim())/100;
						brightness 	= (float)Integer.parseInt(briTextField.getText().trim())/100;

						if(hue < 0) 				hue = 0;
						if(hue > 1.0) 			hue = 1.0f;
						if(saturation < 0) 		saturation = 0;
						if(saturation > 1.0) 	saturation = 1.0f;
						if(brightness < 0) 		brightness = 0;
						if(brightness > 1.0) 	brightness = 1.0f;						

						newColor = Color.getHSBColor(hue,saturation, brightness);
					}
					else if(e.getSource().equals(redTextField) || e.getSource().equals(greenTextField) || e.getSource().equals(blueTextField)) {

						int red 		= Integer.parseInt(redTextField.getText().trim());
						int green 	= Integer.parseInt(greenTextField.getText().trim());
						int blue 	= Integer.parseInt(blueTextField.getText().trim());

						float[] hsb 	= Color.RGBtoHSB(red, green, blue, null);
						hue 			= hsb[0];
						saturation 	= hsb[1];
						brightness 	= hsb[2];

						newColor = Color.getHSBColor(hue,saturation, brightness);		
					}
					else if(e.getSource().equals(cTextField) || e.getSource().equals(mTextField) || e.getSource().equals(yTextField) || e.getSource().equals(kTextField)) {

						float[] cmyk = new float[4];
						cmyk[0] = ((float)Integer.parseInt(cTextField.getText().trim())/100);
						cmyk[1] = ((float)Integer.parseInt(mTextField.getText().trim())/100);
						cmyk[2] = ((float)Integer.parseInt(yTextField.getText().trim())/100);
						cmyk[3] = ((float)Integer.parseInt(kTextField.getText().trim())/100);

						int vrgb = cmykToRgb(cmyk, cmykColorspace);

						Color c = new Color(vrgb);						
						float[] hsb 	= Color.RGBtoHSB(c.getRed(), c.getGreen(), c.getBlue(), null);
						hue 			= hsb[0];
						saturation 	= hsb[1];
						brightness 	= hsb[2];

						newColor = Color.getHSBColor(hue,saturation, brightness);						
					}	

					updateMode();
				}
			}

			@Override
			public void keyReleased(KeyEvent e) {}
		};




		float startY 	= 0.142f;
		float stepY		= 0.025f;

		hueLabel = new FLabel(fframe);
		hueLabel.setBounds(0.208f, startY, 0.020f, 0.023f);
		hueLabel.setFont(baseFont);	hueLabel.setText("H:");
		add(hueLabel);			
		hueTextField = new FTextField(fframe);
		hueTextField.setBounds(0.215f, startY, 0.021f, 0.0232f);
		hueTextField.setBorderColor(Color.BLACK); hueTextField.setBorderRadius(0);hueTextField.setFont(baseFont);hueTextField.setText("0");
		hueTextField.addKeyListener(textFieldKeyListener);
		add(hueTextField);
		satLabel = new FLabel(fframe);
		satLabel.setBounds(0.208f, startY+stepY*1, 0.020f, 0.023f);
		satLabel.setFont(baseFont);	satLabel.setText("S:");
		add(satLabel);			
		satTextField = new FTextField(fframe);
		satTextField.setBounds(0.215f, startY+stepY*1, 0.021f, 0.0232f);
		satTextField.setBorderColor(Color.BLACK); satTextField.setBorderRadius(0);satTextField.setFont(baseFont);satTextField.setText("0");
		satTextField.addKeyListener(textFieldKeyListener);
		add(satTextField);			
		briLabel = new FLabel(fframe);
		briLabel.setBounds(0.208f, startY+stepY*2, 0.020f, 0.023f);
		briLabel.setFont(baseFont);	briLabel.setText("B:");
		add(briLabel);			
		briTextField = new FTextField(fframe);
		briTextField.setBounds(0.215f, startY+stepY*2, 0.021f, 0.0232f);
		briTextField.setBorderColor(Color.BLACK); briTextField.setBorderRadius(0);briTextField.setFont(baseFont);briTextField.setText("0");
		briTextField.addKeyListener(textFieldKeyListener);
		add(briTextField);	


		startY 	= 0.142f;

		hueRadioButton = new FRadioButton(fframe);
		hueRadioButton.setBounds(0.199f, startY + 0.0046f, 0.006f, 0.006f*1.777f);
		hueRadioButton.addMouseListener(new MouseListener() {
			@Override public void mouseClicked(MouseEvent e) {}
			@Override public void mousePressed(MouseEvent e) {setColorPickerMode(0); updateMode();};
			@Override public void mouseReleased(MouseEvent e) {}
			@Override public void mouseEntered(MouseEvent e) {}
			@Override public void mouseExited(MouseEvent e) {}
		});		
		add(hueRadioButton);	

		satRadioButton = new FRadioButton(fframe);
		satRadioButton.setBounds(0.199f, startY + 0.0046f + stepY*1, 0.006f, 0.006f*1.777f);
		satRadioButton.addMouseListener(new MouseListener() {
			@Override public void mouseClicked(MouseEvent e) {}
			@Override public void mousePressed(MouseEvent e) {setColorPickerMode(1); updateMode();};
			@Override public void mouseReleased(MouseEvent e) {}
			@Override public void mouseEntered(MouseEvent e) {}
			@Override public void mouseExited(MouseEvent e) {}
		});
		add(satRadioButton);	

		briRadioButton = new FRadioButton(fframe);
		briRadioButton.setBounds(0.199f, startY + 0.0046f + stepY*2, 0.006f, 0.006f*1.777f);
		briRadioButton.addMouseListener(new MouseListener() {
			@Override public void mouseClicked(MouseEvent e) {}
			@Override public void mousePressed(MouseEvent e) {setColorPickerMode(2); updateMode();};
			@Override public void mouseReleased(MouseEvent e) {}
			@Override public void mouseEntered(MouseEvent e) {}
			@Override public void mouseExited(MouseEvent e) {}
		});
		add(briRadioButton);	




		startY += 0.085f;

		redLabel = new FLabel(fframe);
		redLabel.setBounds(0.208f, startY, 0.020f, 0.023f);
		redLabel.setFont(baseFont);	redLabel.setText("R:");
		add(redLabel);			
		redTextField = new FTextField(fframe);
		redTextField.setBounds(0.215f, startY, 0.021f, 0.0232f);
		redTextField.setBorderColor(Color.BLACK); redTextField.setBorderRadius(0);redTextField.setFont(baseFont);redTextField.setText("0");
		redTextField.addKeyListener(textFieldKeyListener);
		add(redTextField);			
		greenLabel = new FLabel(fframe);
		greenLabel.setBounds(0.208f, startY+stepY*1, 0.020f, 0.023f);
		greenLabel.setFont(baseFont);greenLabel.setText("G:");
		add(greenLabel);			
		greenTextField = new FTextField(fframe);
		greenTextField.setBounds(0.215f, startY+stepY*1, 0.021f, 0.0232f);
		greenTextField.setBorderColor(Color.BLACK); greenTextField.setBorderRadius(0);greenTextField.setFont(baseFont);greenTextField.setText("0");
		greenTextField.addKeyListener(textFieldKeyListener);
		add(greenTextField);			
		blueLabel = new FLabel(fframe);
		blueLabel.setBounds(0.208f, startY+stepY*2, 0.020f, 0.023f);
		blueLabel.setFont(baseFont);	blueLabel.setText("B:");
		add(blueLabel);			
		blueTextField = new FTextField(fframe);
		blueTextField.setBounds(0.215f, startY+stepY*2, 0.021f, 0.0232f);
		blueTextField.setBorderColor(Color.BLACK); blueTextField.setBorderRadius(0);blueTextField.setFont(baseFont);blueTextField.setText("0");
		blueTextField.addKeyListener(textFieldKeyListener);
		add(blueTextField);	

		redRadioButton = new FRadioButton(fframe);
		redRadioButton.setBounds(0.199f, startY + 0.0046f, 0.006f, 0.006f*1.777f);
		redRadioButton.addMouseListener(new MouseListener() {
			@Override public void mouseClicked(MouseEvent e) {}
			@Override public void mousePressed(MouseEvent e) {setColorPickerMode(6); updateMode();};
			@Override public void mouseReleased(MouseEvent e) {}
			@Override public void mouseEntered(MouseEvent e) {}
			@Override public void mouseExited(MouseEvent e) {}
		});
		add(redRadioButton);			
		greenRadioButton = new FRadioButton(fframe);
		greenRadioButton.setBounds(0.199f, startY + 0.0046f + stepY*1, 0.006f, 0.006f*1.777f);
		greenRadioButton.addMouseListener(new MouseListener() {
			@Override public void mouseClicked(MouseEvent e) {}
			@Override public void mousePressed(MouseEvent e) {setColorPickerMode(6); updateMode();};
			@Override public void mouseReleased(MouseEvent e) {}
			@Override public void mouseEntered(MouseEvent e) {}
			@Override public void mouseExited(MouseEvent e) {}
		});
		add(greenRadioButton);			
		blueRadioButton = new FRadioButton(fframe);
		blueRadioButton.setBounds(0.199f, startY + 0.0046f + stepY*2, 0.006f, 0.006f*1.777f);
		blueRadioButton.addMouseListener(new MouseListener() {
			@Override public void mouseClicked(MouseEvent e) {}
			@Override public void mousePressed(MouseEvent e) {setColorPickerMode(6); updateMode();};
			@Override public void mouseReleased(MouseEvent e) {}
			@Override public void mouseEntered(MouseEvent e) {}
			@Override public void mouseExited(MouseEvent e) {}
		});
		add(blueRadioButton);	




		startY -= 0.085f;		

		cLabel = new FLabel(fframe);
		cLabel.setBounds(0.258f, startY, 0.020f, 0.023f);
		cLabel.setFont(baseFont);	cLabel.setText("C:");
		add(cLabel);			
		cTextField = new FTextField(fframe);
		cTextField.setBounds(0.265f, startY, 0.021f, 0.0232f);
		cTextField.setBorderColor(Color.BLACK); cTextField.setBorderRadius(0);cTextField.setFont(baseFont);cTextField.setText("0");
		cTextField.addKeyListener(textFieldKeyListener);
		add(cTextField);
		mLabel = new FLabel(fframe);
		mLabel.setBounds(0.256f, startY+stepY*1, 0.020f, 0.023f);
		mLabel.setFont(baseFont);	mLabel.setText("M:");
		add(mLabel);			
		mTextField = new FTextField(fframe);
		mTextField.setBounds(0.265f, startY+stepY*1, 0.021f, 0.0232f);
		mTextField.setBorderColor(Color.BLACK); mTextField.setBorderRadius(0);mTextField.setFont(baseFont);mTextField.setText("0");
		mTextField.addKeyListener(textFieldKeyListener);
		add(mTextField);			
		yLabel = new FLabel(fframe);
		yLabel.setBounds(0.258f, startY+stepY*2, 0.020f, 0.023f);
		yLabel.setFont(baseFont);	yLabel.setText("Y:");
		add(yLabel);			
		yTextField = new FTextField(fframe);
		yTextField.setBounds(0.265f, startY+stepY*2, 0.021f, 0.0232f);
		yTextField.setBorderColor(Color.BLACK); yTextField.setBorderRadius(0);yTextField.setFont(baseFont);yTextField.setText("0");
		yTextField.addKeyListener(textFieldKeyListener);
		add(yTextField);	
		kLabel = new FLabel(fframe);
		kLabel.setBounds(0.258f, startY+stepY*3, 0.020f, 0.023f);
		kLabel.setFont(baseFont);	kLabel.setText("K:");
		add(kLabel);			
		kTextField = new FTextField(fframe);
		kTextField.setBounds(0.265f,startY+stepY*3, 0.021f, 0.0232f);
		kTextField.setBorderColor(Color.BLACK); kTextField.setBorderRadius(0);kTextField.setFont(baseFont);kTextField.setText("0");
		kTextField.addKeyListener(textFieldKeyListener);
		add(kTextField);	


		label = new JLabel("Select foreground color:");
		label.setVerticalAlignment(JLabel.TOP);
		label.setVerticalTextPosition(JLabel.TOP);
		add(label);	



		// * Label is needed only for fix the placement bug for the Color Panels (without the label the Panels are not in place!) *
		JLabel labelLeer = new JLabel("");
		labelLeer.setBounds(0,0,1,1);
		add(labelLeer);	


		if(initColor == null) {

			if(colorBoxClicked == 1) 	newColor = getForegroundColor();
			else							newColor = getBackgroundColor();

			currentColor 	= newColor;
		}
		else {
			
			newColor = initColor;
			currentColor = newColor;
		}

		setResizable(false);
		setVisible(true);
		setAlwaysOnTop(true);

		updateArrowSize();


		float[] hsb = new float[3];
		Color.RGBtoHSB(newColor.getRed(), newColor.getGreen(), newColor.getBlue(), hsb);

		hue 			= hsb[0];
		saturation 	= hsb[1];
		brightness 	= hsb[2];


		updateMode();

		updateUIElements();

		repaint();				
	}


	public void setColorPickerMode(int mode) {

		colorPickerMode = mode;	
		prefs.putInt(PREF_NAME_colorpicker_mode, colorPickerMode);				
	}

	public int getColorPickerMode() {
		return colorPickerMode;
	}


	public void setForegroundColor(Color color) {

		foregroundColor = color;

		String c = encodeColorToString(color);		
		prefs.put(PREF_NAME_foreground_color, c);	
	}

	public void setBackgroundColor(Color color) {

		backgroundColor = color;

		String c = encodeColorToString(color);		
		prefs.put(PREF_NAME_background_color, c);				
	}

	public Color getForegroundColor() {		
		return foregroundColor;
	}

	public Color getBackgroundColor() {		
		return backgroundColor;
	}


	public String encodeColorToString(Color color) {

		String hex = String.format("#%02x%02x%02x", color.getRed(), color.getGreen(), color.getBlue());  
		return hex;
	}

	public int cmykToRgb(float[] cmyk, ColorSpace rgbColorspace) {

		int rgb = 0;

		try {
			float[] fRGB = rgbColorspace.toRGB(cmyk);			
			rgb =  (int)(fRGB[0]*255) << 16 | (int)(fRGB[1]*255) << 8 |  (int)(fRGB[2]*255);		
		}
		catch(Exception e) { e.printStackTrace();}

		return rgb;
	}


	static public float[] rgbToCmyk(int rgb, ColorSpace cmykColorspace) {

		float[] fromRGB = new float[4];

		try {

			float[] c = new float[3];
			c[0] = (1.0f/255)*((rgb >> 16) & 0xff);
			c[1] = (1.0f/255)*((rgb >> 8) & 0xff);
			c[2] = (1.0f/255)*((rgb) & 0xff);

			fromRGB = cmykColorspace.fromRGB(c);			
		}
		catch(Exception e) { e.printStackTrace();}

		return fromRGB;
	}


	public void monitorResolutionChanged() {

		updateUIElements();
	}


	public void updateUIElements() {

		width 		= fframe.monitor.convertFloatToRealWidth(0.31f);
		height 		= fframe.monitor.convertFloatToRealHeight(0.35f);		
		setBounds(fframe.getX() + fframe.getWidth()/2-width/2, fframe.getY() + (int)(fframe.getHeight()/2.5f)-height/2,width, height);

		updateArrowSize();

		colorPanelXPos 	= fframe.monitor.convertFloatToRealWidth(0.015f);
		colorPanelYPos 	= fframe.monitor.convertFloatToRealHeight(0.045f);	
		colorPanel.setBounds(colorPanelXPos, colorPanelYPos,colorImage.getWidth(),colorImage.getHeight());		

		colorVerticalPanelXPos 	= fframe.monitor.convertFloatToRealWidth(0.160f);
		colorVerticalPanelYPos 	= fframe.monitor.convertFloatToRealHeight(0.045f);		
		colorVerticalPanelWidth 	= verticalImageWidth  +  fframe.monitor.convertFloatToRealWidth(0.02f);
		colorVerticalPanelHeight	= verticalImageHeight + colorArrowSize*2;	
		colorVerticalPanel.setBounds(colorVerticalPanelXPos, colorVerticalPanelYPos-colorArrowSize, colorVerticalPanelWidth, colorVerticalPanelHeight);		

		colorsNewCurrentXPos 	= fframe.monitor.convertFloatToRealWidth(0.200f);
		colorsNewCurrentYPos 	= fframe.monitor.convertFloatToRealHeight(0.045f);	
		colorsNewCurrentWidth 	= fframe.monitor.convertFloatToRealWidth(0.032f);
		colorsNewCurrentHeight 	= fframe.monitor.convertFloatToRealHeight(0.062f); 
		colorsNewCurrent.setBounds(colorsNewCurrentXPos, colorsNewCurrentYPos, colorsNewCurrentWidth,colorsNewCurrentHeight);		

		int labelX 				= fframe.monitor.convertFloatToRealWidth(0.016f);
		int labelY 				= fframe.monitor.convertFloatToRealHeight(0.022f);
		int labelWidth 			= fframe.monitor.convertFloatToRealWidth(0.15f);
		int labelHeight 			= fframe.monitor.convertFloatToRealHeight(0.03f);

		updateFont();

		if(colorBoxClicked == 1) 	label.setText("Select foreground color:");
		else 						label.setText("Select background color:");
		label.setFont(baseFont);
		label.setBounds(labelX,labelY,labelWidth,labelHeight);
	}



	// ************************************
	// ***                              ***
	// ***         Mouse Events         ***
	// ***                              ***
	// ************************************

	private void mouseEventColorPanel(MouseEvent e) {

		int x = e.getX();
		int y = e.getY();
		if(x < 0) x = 0;
		if(x > colorImage.getWidth()-1) x = colorImage.getWidth()-1;
		if(y < 0) y = 0;
		if(y > colorImage.getHeight()-1) y = colorImage.getHeight()-1;

		if(colorImageType == COLOR_IMAGE_TYPE_CIRCLE) 			circleMouseMoved(x,y);

		if(colorImageType == COLOR_IMAGE_TYPE_HUE_BOX) 			hueMouseMoved(x,y);
		if(colorImageType == COLOR_IMAGE_TYPE_SATURATION_BOX) 	saturationMouseMoved(x,y);
		if(colorImageType == COLOR_IMAGE_TYPE_BRIGHTNESS_BOX) 	brightnessMouseMoved(x,y);
	}

	private void mouseEventColorVerticalPanel(MouseEvent e) {

		int x = e.getX();
		int y = e.getY() - colorArrowSize;
		if(x < 0) x = 0;
		if(x > verticalImageWidth) x = verticalImageWidth-1;
		if(y < 0) y = 0;
		if(y > verticalImageHeight) y = verticalImageHeight-1;

		if(colorImageType == COLOR_IMAGE_TYPE_CIRCLE) 			circleSliderMouseMoved(x,y);

		if(colorImageType == COLOR_IMAGE_TYPE_HUE_BOX) 			hueSliderMouseMoved(x,y);
		if(colorImageType == COLOR_IMAGE_TYPE_SATURATION_BOX) 	saturationSliderMouseMoved(x,y);
		if(colorImageType == COLOR_IMAGE_TYPE_BRIGHTNESS_BOX) 	brightnessSliderMouseMoved(x,y);
	}




	// ************************************
	// ***                              ***
	// ***             HUE              ***
	// ***                              ***
	// ************************************

	private void hueMouseMoved(int x, int y) {

		// * Ring Position Update *
		colorRingXPos = x;
		colorRingYPos = y;

		// * Update Current/New Color Box *
		saturation = ((1.0f/colorImage.getWidth())*x);	
		brightness = 1.0f-((1.0f/colorImage.getHeight())*y);	

		if(x == colorImage.getWidth()-1) 	saturation = 1; 	// * the last value is only "near" 1.0, set it to absolute "1.0" *
		if(y == colorImage.getHeight()-1) 	brightness = 0; 	// * the last value is only "near" 1.0, set it to absolute "1.0" *


		newColor = Color.getHSBColor(hue,saturation, brightness);
		updateTextFields();

		repaint();
	}

	private void hueSliderMouseMoved(int x, int y) {

		// * Move Arrow Vertical Color Panel *
		colorArrowYPos = colorArrowSize + y;

		// * Update Left Color Panel *
		hue = 1.0f-((1.0f/colorImage.getHeight())*y);
		createLeftColorImage();

		// * Update Current/New Color Box  + TextFields *	
		newColor = Color.getHSBColor(hue,saturation, brightness);
		updateTextFields();

		repaint();
	}

	public BufferedImage createHueColorBoxImage(int width, int height) {

		BufferedImage image 	= new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
		Graphics2D g2 		= (Graphics2D)image.getGraphics();

		int rgb = 0;		
		for(int b=0;b<height;b++) {
			for(int s=0;s<width;s++) {	
				rgb = Color.HSBtoRGB(hue, ((1.0f/width)*s), 1.0f-((1.0f/height)*b));				
				g2.setColor(new Color(rgb));
				g2.drawLine(s, b, s, b);
			}
		}		
		return image;
	}

	public BufferedImage createVerticalColorHueBoxImage(int width, int height) {

		BufferedImage image 	= new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
		Graphics2D g2 		= (Graphics2D)image.getGraphics();

		int rgb = 0;		
		for(int h=0;h<height;h++) {
			rgb = Color.HSBtoRGB(1.0f-((1.0f/height)*h), 1.0f, 1.0f);				
			g2.setColor(new Color(rgb));
			g2.drawLine(0, h, width, h);
		}		

		return image;
	}


	// ************************************
	// ***                              ***
	// ***         SATURATION           ***
	// ***                              ***
	// ************************************

	private void saturationMouseMoved(int x, int y) {

		// * Ring Position Update *
		colorRingXPos 	= x;
		colorRingYPos 	= y;

		hue 				= ((1.0f/colorImage.getWidth())*x);	
		brightness 		= 1.0f-((1.0f/colorImage.getHeight())*y);			

		if(x == colorImage.getWidth()-1) 	hue = 0; 	// * the last value is only "near" 1.0, set it to absolute "1.0" *
		if(y == colorImage.getHeight()-1) 	brightness = 0; 	// * the last value is only "near" 1.0, set it to absolute "1.0" *

		createVerticalColorImage();


		// * Update Current/New Color Box  + TextFields *	
		newColor = Color.getHSBColor(hue,saturation, brightness);
		updateTextFields();

		repaint();
	}

	private void saturationSliderMouseMoved(int x, int y) {

		// * Move Arrow Vertical Color Panel *
		colorArrowYPos = colorArrowSize + y;

		// * Update Left Color Panel *
		saturation = 1.0f-((1.0f/colorImage.getHeight())*y);
		createLeftColorImage();


		// * Update Current/New Color Box  + TextFields *
		newColor = Color.getHSBColor(hue,saturation, brightness);
		updateTextFields();

		repaint();
	}

	public BufferedImage createSaturationColorBoxImage(int width, int height) {

		BufferedImage image 	= new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
		Graphics2D g2 		= (Graphics2D)image.getGraphics();

		int rgb = 0;		
		for(int b=0;b<height;b++) {
			for(int s=0;s<width;s++) {	

				rgb = Color.HSBtoRGB(((1.0f/width)*s), saturation, 1.0f-((1.0f/height)*b));				
				g2.setColor(new Color(rgb));
				g2.drawLine(s, b, s, b);
			}
		}
		return image;
	}

	public BufferedImage createVerticalSaturationColorBoxImage(int width, int height) {

		BufferedImage image 	= new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
		Graphics2D g2 		= (Graphics2D)image.getGraphics();

		int rgb = 0;		
		for(int s=0;s<height;s++) {

			rgb = Color.HSBtoRGB(hue, 1.0f-((1.0f/height)*s), brightness);				
			g2.setColor(new Color(rgb));
			g2.drawLine(0, s, width, s);

		}
		return image;
	}




	// ************************************
	// ***                              ***
	// ***         BRIGHTNESS           ***
	// ***                              ***
	// ************************************

	private void brightnessMouseMoved(int x, int y) {

		// * Ring Position Update *
		colorRingXPos 	= x;
		colorRingYPos 	= y;

		hue 				= ((1.0f/colorImage.getWidth())*x);	
		saturation 		= 1.0f-((1.0f/colorImage.getHeight())*y);			

		if(x == colorImage.getWidth()-1) 	hue = 0; 	// * the last value is only "near" 1.0, set it to absolute "1.0" *
		if(y == colorImage.getHeight()-1) 	saturation = 0; 	// * the last value is only "near" 1.0, set it to absolute "1.0" *

		createVerticalColorImage();


		// * Update Current/New Color Box  + TextFields *	
		newColor = Color.getHSBColor(hue,saturation, brightness);
		updateTextFields();

		repaint();
	}

	private void brightnessSliderMouseMoved(int x, int y) {

		// * Move Arrow Vertical Color Panel *
		colorArrowYPos = colorArrowSize + y;

		// * Update Left Color Panel *
		brightness = 1.0f-((1.0f/colorImage.getHeight())*y);
		createLeftColorImage();


		// * Update Current/New Color Box  + TextFields *	
		newColor = Color.getHSBColor(hue,saturation, brightness);
		updateTextFields();

		repaint();
	}

	public BufferedImage createBrightnessColorBoxImage(int width, int height) {

		BufferedImage image 	= new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
		Graphics2D g2 		= (Graphics2D)image.getGraphics();

		int rgb = 0;		
		for(int b=0;b<height;b++) {
			for(int s=0;s<width;s++) {	

				rgb = Color.HSBtoRGB(((1.0f/width)*s), 1.0f-((1.0f/height)*b), brightness);				
				g2.setColor(new Color(rgb));
				g2.drawLine(s, b, s, b);
			}
		}
		return image;
	}

	public BufferedImage createVerticalBrightnessColorBox(int width, int height) {

		BufferedImage image 	= new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
		Graphics2D g2 		= (Graphics2D)image.getGraphics();

		int rgb = 0;		
		for(int s=0;s<height;s++) {

			rgb = Color.HSBtoRGB(hue, saturation, 1.0f-((1.0f/height)*s));				
			g2.setColor(new Color(rgb));
			g2.drawLine(0, s, width, s);

		}
		return image;
	}



	// ************************************
	// ***                              ***
	// ***           RGB Circle         ***
	// ***                              ***
	// ************************************

	private void circleMouseMoved (int x, int y) {

		int[] newPos = getCirclePos(colorImage.getWidth()/2, colorImage.getWidth()/2, colorImage.getHeight()/2, x, y);
		x = newPos[0];
		y = newPos[1];

		// * Ring Position Update *
		colorRingXPos = x;
		colorRingYPos = y;

		int c = colorImage.getRGB(x, y);

		float[] hsb = Color.RGBtoHSB(((c >> 16) & 0xff), ((c >> 8) & 0xff), ((c) & 0xff), null);

		hue 			= hsb[0];
		saturation 	= hsb[1];		
		createVerticalColorImage();


		// * Update Current/New Color Box  + TextFields *		
		newColor = new Color(c);		
		updateTextFields();

		repaint();
	}


	private void circleSliderMouseMoved(int x, int y) {

		// * Move Arrow Vertical Color Panel *
		colorArrowYPos = colorArrowSize + y;

		// * Update Left Color Panel *
		brightness = 1.0f-((1.0f/colorImage.getHeight())*y);
		createLeftColorImage();


		// * Update Current/New Color Box  + TextFields *	
		newColor = Color.getHSBColor(hue,saturation, brightness);
		updateTextFields();

		repaint();
	}




	public BufferedImage createColorPickerCircleImage(int size) {

		float[] sinusWaveFunction = new float[360];
		for(int i=0;i<360;i++) sinusWaveFunction[i] = (float) (1.0f+ 0.6f * Math.sin(i*(3.1415926/30) - 2.5f));


		float sum = 0;	// 0-360	
		int pos = 305;		

		float[] colorTable = new float[360];		
		for(int i=0;i<360;i++) {

			sum += sinusWaveFunction[pos];
			colorTable[i] =  sum /360.0f;			

			pos++;
			if(pos >= 360) pos = 0;			
		}

		float hue = 1;
		float sat = 1;


		float radius = ((float)size)/2f;

		// * Number of pixels to antialias *
		float k = 1.2f;


		BufferedImage image = new BufferedImage(size, size, BufferedImage.TYPE_INT_ARGB);
		int[] row = new int[size];

		for(int y = 0; y<size; y++) {

			float y2 = (y-size/2f);			
			for(int x = 0; x<size; x++) {

				float x2 = (x-size/2f);
				double theta = Math.atan2(y2,x2)-2*Math.PI/2.0;
				if(theta<0) theta += 2*Math.PI;

				double r = Math.sqrt(x2*x2+y2*y2);
				if(r<=radius) {

					hue = ((float)(theta/(2*Math.PI)));
					sat = (float)(r/radius);
					hue = (((hue*360) % 360) + 360) % 360; // normalize 360 > legacyHue >= 0

					row[size-1-x] = Color.HSBtoRGB(colorTable[(int) hue], sat, brightness);
					if(r>radius-k) {

						int alpha = (int)(255-255*(r-radius+k)/k);
						if(alpha<0) 		alpha = 0;
						if(alpha>255) 	alpha = 255;

						row[size-1-x] = row[size-1-x] & 0xffffff+(alpha << 24);
					}
				} 
				else row[size-1-x] = 0x00000000;					
			}
			image.getRaster().setDataElements(0, y, size, 1, row);
		}
		return image;
	}

	public BufferedImage createVerticalColorCircleBoxImage(int width, int height) {

		BufferedImage image 	= new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
		Graphics2D g2 		= (Graphics2D)image.getGraphics();

		int rgb = 0;		
		for(int s=0;s<height;s++) {

			rgb = Color.HSBtoRGB(hue, saturation, 1.0f-((1.0f/height)*s));				
			g2.setColor(new Color(rgb));
			g2.drawLine(0, s, width, s);

		}
		return image;
	}





	// ************************************
	// ***                              ***
	// ***             UPDATE           ***
	// ***                              ***
	// ************************************

	private void updateMode() {

		hueRadioButton.setSelected(false);
		satRadioButton.setSelected(false);
		briRadioButton.setSelected(false);
		redRadioButton.setSelected(false);
		greenRadioButton.setSelected(false);
		blueRadioButton.setSelected(false);

		if(getColorPickerMode() == 0) {

			colorImageType = COLOR_IMAGE_TYPE_HUE_BOX;
			hueRadioButton.setSelected(true);
		}
		if(getColorPickerMode() == 1) {

			colorImageType = COLOR_IMAGE_TYPE_SATURATION_BOX;
			satRadioButton.setSelected(true);
		}
		if(getColorPickerMode() == 2) {

			colorImageType = COLOR_IMAGE_TYPE_BRIGHTNESS_BOX;
			briRadioButton.setSelected(true);
		}
		if(getColorPickerMode() == 3) {

			colorImageType = COLOR_IMAGE_TYPE_RED_BOX;
			redRadioButton.setSelected(true);
		}
		if(getColorPickerMode() == 4) {

			colorImageType = COLOR_IMAGE_TYPE_GREEN_BOX;
			greenRadioButton.setSelected(true);
		}
		if(getColorPickerMode() == 5) {

			colorImageType = COLOR_IMAGE_TYPE_BLUE_BOX;
			blueRadioButton.setSelected(true);
		}
		if(getColorPickerMode() == 6) {

			colorImageType = COLOR_IMAGE_TYPE_CIRCLE;			
			redRadioButton.setSelected(true);
			greenRadioButton.setSelected(true);
			blueRadioButton.setSelected(true);
		}		

		updateColorBoxes();
		updateRingPositionLeftColorPanel();
		updateArrowPositionVerticalColorPanel();

		updateTextFields();

		repaint();	
	}

	private void updateColorBoxes() {

		createLeftColorImage();
		createVerticalColorImage();		
	}

	private void updateRingPositionLeftColorPanel() {

		if(colorImageType == COLOR_IMAGE_TYPE_HUE_BOX) {

			colorRingXPos = (int) (((float)colorImage.getWidth()/1.0f)*saturation);
			colorRingYPos = (int) (((float)colorImage.getHeight()/1.0f)*(1.0f-brightness));

			if(colorRingXPos < 0) colorRingXPos = 0;
			if(colorRingXPos > colorImage.getWidth()) colorRingXPos = colorImage.getWidth()-1;
			if(colorRingYPos < 0) colorRingYPos = 0;
			if(colorRingYPos > colorImage.getHeight()) colorRingYPos = colorImage.getHeight()-1;			
		}
		else if(colorImageType == COLOR_IMAGE_TYPE_SATURATION_BOX) {

			colorRingXPos = (int) (((float)colorImage.getWidth()/1.0f)*hue);
			colorRingYPos = (int) (((float)colorImage.getHeight()/1.0f)*(1.0f-brightness));

			if(colorRingXPos < 0) colorRingXPos = 0;
			if(colorRingXPos > colorImage.getWidth()) colorRingXPos = colorImage.getWidth()-1;
			if(colorRingYPos < 0) colorRingYPos = 0;
			if(colorRingYPos > colorImage.getHeight()) colorRingYPos = colorImage.getHeight()-1;		
		}
		else if(colorImageType == COLOR_IMAGE_TYPE_BRIGHTNESS_BOX) {

			colorRingXPos = (int) (((float)colorImage.getWidth()/1.0f)*hue);
			colorRingYPos = (int) (((float)colorImage.getHeight()/1.0f)*(1.0f-saturation));

			if(colorRingXPos < 0) colorRingXPos = 0;
			if(colorRingXPos > colorImage.getWidth()) colorRingXPos = colorImage.getWidth()-1;
			if(colorRingYPos < 0) colorRingYPos = 0;
			if(colorRingYPos > colorImage.getHeight()) colorRingYPos = colorImage.getHeight()-1;		
		}

		else if(colorImageType == COLOR_IMAGE_TYPE_CIRCLE) {

			int[] colorPos = findNearestColorPosition(colorImage, Color.getHSBColor(hue, saturation, brightness).getRGB());
			colorRingXPos = colorPos[0];
			colorRingYPos = colorPos[1];

		}		
	}

	private void updateArrowPositionVerticalColorPanel() {

		if(colorImageType == COLOR_IMAGE_TYPE_HUE_BOX) {

			colorArrowYPos = colorArrowSize + (int) (((float)colorImage.getHeight()/1.0f)*(1.0f-hue));

			if(colorArrowYPos < 0) colorArrowYPos = 0;
			if(colorArrowYPos > colorImage.getHeight()) colorArrowYPos = colorImage.getHeight()-1;			
		}
		else if(colorImageType == COLOR_IMAGE_TYPE_SATURATION_BOX) {

			colorArrowYPos = colorArrowSize + (int) (((float)colorImage.getHeight()/1.0f)*(1.0f-saturation));

			if(colorArrowYPos < 0) colorArrowYPos = 0;
			if(colorArrowYPos > colorImage.getHeight()) colorArrowYPos = colorImage.getHeight()-1;	
		}
		else if(colorImageType == COLOR_IMAGE_TYPE_BRIGHTNESS_BOX) {

			colorArrowYPos = colorArrowSize + (int) (((float)colorImage.getHeight()/1.0f)*(1.0f-brightness));

			if(colorArrowYPos < 0) colorArrowYPos = 0;
			if(colorArrowYPos > colorImage.getHeight()) colorArrowYPos = colorImage.getHeight()-1;	
		}		
		else if(colorImageType == COLOR_IMAGE_TYPE_CIRCLE) {

			colorArrowYPos = colorArrowSize + (int) (((float)colorImage.getHeight()/1.0f)*(1.0f-brightness));

			if(colorArrowYPos < 0) colorArrowYPos = 0;
			if(colorArrowYPos > colorImage.getHeight()) colorArrowYPos = colorImage.getHeight()-1;	
		}
	}

	private void updateTextFields() {

		Color rgb = Color.getHSBColor(hue,saturation, brightness);			

		hueTextField.setText("" + (int)((360.0f/1.0f)*hue));		
		satTextField.setText("" + (int)((100.0f/1.0f)*saturation));
		briTextField.setText("" + (int)((100.0f/1.0f)*brightness));
		redTextField.setText("" + rgb.getRed());
		greenTextField.setText("" + rgb.getGreen());
		blueTextField.setText("" +rgb.getBlue());

		float[] cmykColor = rgbToCmyk(Color.getHSBColor(hue,saturation, brightness).getRGB(), cmykColorspace);
		cTextField.setText("" + (int)(cmykColor[0]*100.0f));
		mTextField.setText("" + (int)(cmykColor[1]*100.0f));
		yTextField.setText("" + (int)(cmykColor[2]*100.0f));
		kTextField.setText("" + (int)(cmykColor[3]*100.0f));
	}	


	private void updateArrowSize() {

		colorArrowSize 	= fframe.monitor.convertFloatToRealWidth(0.0035f);
	}

	private void updateFont() {

		baseFont = new Font("pix Chicago", Font.PLAIN,  fframe.monitor.convertFloatToRealHeight(0.009f));
	}	




	// ************************************
	// ***                              ***
	// ***         Create Images        ***
	// ***                              ***
	// ************************************

	private void createLeftColorImage() {

		int width 	= fframe.monitor.convertFloatToRealWidth(0.14f);
		int height 	= fframe.monitor.convertFloatToRealHeight(0.25f);

		if(colorImageType == COLOR_IMAGE_TYPE_CIRCLE) 			colorImage = createColorPickerCircleImage(width-1);	

		if(colorImageType == COLOR_IMAGE_TYPE_HUE_BOX) 			colorImage = createHueColorBoxImage(width-1,height-1);		
		if(colorImageType == COLOR_IMAGE_TYPE_SATURATION_BOX) 	colorImage = createSaturationColorBoxImage(width-1,height-1);		
		if(colorImageType == COLOR_IMAGE_TYPE_BRIGHTNESS_BOX) 	colorImage = createBrightnessColorBoxImage(width-1,height-1);
	}

	private void createVerticalColorImage() {

		verticalImageWidth 	= fframe.monitor.convertFloatToRealWidth(0.014f);
		verticalImageHeight 	= fframe.monitor.convertFloatToRealHeight(0.25f);

		if(colorImageType == COLOR_IMAGE_TYPE_CIRCLE) 			verticalColorImage = createVerticalColorCircleBoxImage(verticalImageWidth-1,verticalImageHeight-1);	

		if(colorImageType == COLOR_IMAGE_TYPE_HUE_BOX) 			verticalColorImage = createVerticalColorHueBoxImage(verticalImageWidth-1,verticalImageHeight-1);		
		if(colorImageType == COLOR_IMAGE_TYPE_SATURATION_BOX) 	verticalColorImage = createVerticalSaturationColorBoxImage(verticalImageWidth-1,verticalImageHeight-1);		
		if(colorImageType == COLOR_IMAGE_TYPE_BRIGHTNESS_BOX) 	verticalColorImage = createVerticalBrightnessColorBox(verticalImageWidth-1,verticalImageHeight-1);
	}




	public int[] getCirclePos(float circleRadius, int originX, int originY, int touchPosX, int touchPosY) {

		float dx,dy,theta,c,newX = 0,newY = 0;

		int[] newPos = {touchPosX,touchPosY};

		dx 		= (touchPosX - originX);
		dy 		= (touchPosY - originY);
		c 		= (float) Math.sqrt(dx*dx + dy*dy);
		theta 	= (float) Math.atan(Math.abs(dy/dx));

		if (c <= circleRadius) return newPos;
		else if(c <= circleRadius*1.5) {

			if(dx>0 && dy<0) { //top right quadrant
				newX = (float) (circleRadius * Math.cos(theta));
				newY = (float) -(circleRadius * Math.sin(theta));
			}

			else if(dx<0 && dy<0) { //top left quadrant
				newX = (float) -(circleRadius * Math.cos(theta));
				newY = (float) -(circleRadius * Math.sin(theta));
			}

			else if(dx<0 && dy>0) { //bottom left quadrant
				newX = (float) -(circleRadius * Math.cos(theta));
				newY = (float) (circleRadius * Math.sin(theta));
			}

			else if(dx > 0 && dy > 0){ //bottom right quadrant
				newX = (float) (circleRadius * Math.cos(theta));
				newY = (float) (circleRadius * Math.sin(theta));
			}

			newPos[0] = (originX + (int)newX);
			newPos[1] = (originY + (int)newY);
		}
		return newPos;
	}

	private int[] findNearestColorPosition(BufferedImage image, int colorRGB) {

		int sR = ((colorRGB >> 16) & 0xff);
		int sG = ((colorRGB >> 8) & 0xff);
		int sB = (colorRGB & 0xff);

		int colorSum = 255*255*255; 

		int[] pos = new int[2];

		for(int y=0;y<image.getHeight();y++) {
			for(int x=0;x<image.getWidth();x++) {

				int rgb = image.getRGB(x, y);				
				int iR 	= ((rgb >> 16) & 0xff);
				int iG 	= ((rgb >> 8) & 0xff);
				int iB 	= (rgb & 0xff);

				int sum = Math.abs(iR-sR)  + Math.abs(iG-sG) + Math.abs(iB-sB);
				if(sum < colorSum) {

					colorSum = sum;
					pos[0] = x;
					pos[1] = y;
				}				
			}
		}

		return pos;
	}
}