package converter.tools;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.event.MouseEvent;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.awt.geom.AffineTransform;
import java.awt.geom.Ellipse2D;

import javax.swing.JPanel;

public class ZoomPanel extends JPanel implements MouseWheelListener {

	private static final int DIMENSION = 300;
	private static double ZOOM = 8;

	public Shape s;

	public ZoomPanel() {
		setLayout(new GridLayout());    
		
		addMouseWheelListener(this);
	}

	@Override
	public void paint(final Graphics g) {
		super.paint(g);

		if (s != null) {

			final Graphics2D g2 = (Graphics2D) g.create();
			g2.setClip(s);
			final Rectangle b = s.getBounds();
			g2.clearRect(b.x, b.y, b.width, b.height);
			final AffineTransform transform = new AffineTransform();
			final double centerX = b.getCenterX();
			final double centerY = b.getCenterY();
			transform.translate(centerX, centerY);
			transform.scale(ZOOM, ZOOM);
			transform.translate(-centerX, -centerY);
			g2.setTransform(transform);
			//  g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
			// g2.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);
			super.paint(g2);
			g2.setTransform(new AffineTransform());
			g2.setClip(g.getClip());
			g2.draw(s);
			g2.dispose();
		}
	}


	public void moveShape(final MouseEvent e) {
		final Point p = e.getPoint();
		final int hd = DIMENSION / 2;
		s = new Ellipse2D.Double(p.x - hd, p.y - hd, DIMENSION, DIMENSION);
		repaint();
	}

	@Override
	public void mouseWheelMoved(MouseWheelEvent e) {
		
		if(s != null) {
			if(e.getWheelRotation() < 0) {

				if(ZOOM < 16) ZOOM++;
			}
			else if(e.getWheelRotation() > 0) {

				if(ZOOM > 2) ZOOM--;
			}
			
			repaint();
		}
	}



}