package ffilechooser;

import java.awt.AWTEvent;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.Toolkit;
import java.awt.event.AWTEventListener;
import java.util.prefs.Preferences;

import javax.swing.AbstractButton;
import javax.swing.BorderFactory;
import javax.swing.ButtonModel;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.UIManager;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import ffilechooser.FMonitor.MonitorListener;

public class FCheckBox extends JCheckBox {

	float x 			= 0;
	float y 			= 0;
	float width 		= 0.02f;
	float height 	= 0.01f;

	String fontName 	= "Arial";
	int fontStyle 	= Font.PLAIN;
	float fontSize 	= 0.01f;

	Color fontColor = Color.DARK_GRAY;

	float monitorOldWidth = 0;
	float monitorOldHeight = 0;


	// * Preferences *
	static Preferences prefs = Preferences.userNodeForPackage(FCheckBox.class);
	final static String PREF_NAME_checked_state	= "checked_state";

	boolean state = false;



	FFrame fframe;

	public FCheckBox(final FFrame fframe) {

		this.fframe = fframe;

		Toolkit.getDefaultToolkit().addAWTEventListener(new AWTEventListener() {

			@Override
			public void eventDispatched(AWTEvent event) {

				if(fframe.monitor.width != monitorOldWidth || fframe.monitor.height != monitorOldHeight) {

					monitorOldWidth = fframe.monitor.width;
					monitorOldHeight = fframe.monitor.height;
					updateBounds();
					updateFont();
				}
			}
		}, AWTEvent.PAINT_EVENT_MASK);


		addChangeListener(new ChangeListener() {

			@Override
			public void stateChanged(ChangeEvent e) {

				AbstractButton abstractButton = (AbstractButton)changeEvent.getSource();
				ButtonModel buttonModel = abstractButton.getModel();
				boolean selected = buttonModel.isSelected();
								
				prefs.putBoolean(PREF_NAME_checked_state, selected);	
			}
		});

		state = prefs.getBoolean(PREF_NAME_checked_state, state);	
		
		setSelected(state);
	}

	private void updateFont() {

		Font font 	= new Font(fontName, fontStyle, fframe.monitor.convertFloatToRealHeight(fontSize));	
		super.setFont(font);	
		repaint();		
	}


	public void setText(String s) {
		super.setText(s);		
	}	

	public void setFont(Font font) {				
		super.setFont(font);

		if(fframe == null) return;

		fontName 	= font.getName();
		fontStyle 	= font.getStyle();
		fontSize 	= fframe.monitor.convertRealToFloatHeight(font.getSize());
	}	

	public void setFontSize(float size) {

		this.fontSize = size;
		updateFont();
	}

	public void setFontName(String fontName) {

		this.fontName = fontName;
		updateFont();
	}

	public void setFontStyle(int fontStyle) {

		this.fontStyle = fontStyle;
		updateFont();
	}

	public void setFontColor(Color color) {
		this.fontColor = color;
	}

	public void setBounds(float x, float y, float width, float height) {

		this.x 		= x;
		this.y 		= y;
		this.width 	= width;
		this.height 	= height;	
		updateBounds();
	}

	public void setLocation(float x, float y) {

		this.x 		= x;
		this.y 		= y;		
		updateBounds();
	}

	public void setSize(float width, float height) {

		this.width 	= width;
		this.height 	= height;	
		updateBounds();
	}

	private void updateBounds() {
		
		setBounds(fframe.monitor.convertFloatToRealWidth(x), fframe.monitor.convertFloatToRealHeight(y), fframe.monitor.convertFloatToRealWidth(width), fframe.monitor.convertFloatToRealHeight(height));
	}

	@Override 
	protected void paintComponent(Graphics g) {

		if (!isOpaque()) {

			Graphics2D g2 = (Graphics2D) g.create();

			g2.setFont(getFont());
			int textHeight 	= g2.getFontMetrics().getHeight();
			int textAsc 		= g2.getFontMetrics().getDescent();

			setBounds(fframe.monitor.convertFloatToRealWidth(x), fframe.monitor.convertFloatToRealHeight(y), fframe.monitor.convertFloatToRealWidth(width), textHeight + textAsc);

			g2.dispose();
		}

		super.setForeground(fontColor);
		super.paintComponent(g);
	}
}
