package ffilechooser;

import java.awt.AWTEvent;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.AWTEventListener;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.ListCellRenderer;
import javax.swing.SwingUtilities;
import javax.swing.border.Border;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.LineBorder;
import javax.swing.plaf.basic.BasicComboBoxUI;
import javax.swing.plaf.basic.BasicComboPopup;
import javax.swing.plaf.basic.ComboPopup;

import ffilechooser.FMonitor.MonitorListener;


public class FComboBox extends JComboBox {


	float x 					= 0;
	float y 					= 0;
	float width 				= 0.02f;
	float height 			= 0.01f;
	float radius 			= 0.005f;	
	float strokeSize 		= 0.001f;

	Color borderColor  		= Color.GRAY;

	String fontName 			= "Arial";
	int fontStyle 			= Font.PLAIN;
	float fontSize 			= 0.01f;


	float monitorOldWidth = 0;
	float monitorOldHeight = 0;
	
	
	FFrame fframe;
	JComboBox comboBox;

	String[] list;

	// * "-" has everytime the first Seperator index, so if a seperator is selected, then take the last hovered index *
	int lastHoveringIndex = 0;


	@SuppressWarnings({ "unchecked", "rawtypes" })
	public FComboBox(final FFrame fframe, final String[] list) {

		super(list);				

		this.fframe 					= fframe;
		this.list 					= list;
		comboBox 					= this;


		setUI(new BasicComboBoxUI() {

			@Override
			protected ComboPopup createPopup() {
				BasicComboPopup basicComboPopup = new BasicComboPopup(comboBox);					
				// * Draw Border *
				basicComboPopup.setBorder(new LineBorder(new Color(129,129,129)));
				return basicComboPopup;
			}

			@Override
			protected JButton createArrowButton() {
				return null; // * Draw no Button here *
			}

			@Override
			public void paintCurrentValue(Graphics g, Rectangle bounds, boolean hasFocus) {
				// * Draw no Text here *
			}

			@Override
			public void paint(Graphics g, JComponent c) {

				Graphics2D g2d = (Graphics2D) g.create();

				int r 			= fframe.monitor.convertFloatToRealHeight(radius);
				int stroke 		= fframe.monitor.convertFloatToRealHeight(strokeSize);
				int stringGap 	= fframe.monitor.convertFloatToRealHeight(0.008f);

				// * Draw Frame *
				g2d.setStroke(new BasicStroke(stroke));
				g2d.setColor(getBackground());
				g2d.fillRoundRect(1, 1, c.getWidth()-2, c.getHeight()-2,r,r);
				g2d.setColor(borderColor);
				g2d.drawRoundRect(1, 1, c.getWidth()-2, c.getHeight()-2,r,r);


				// * Calculate Sizes *
				FontMetrics fm 			= getFontMetrics(getFont());	
				int textYPos 			= getHeight()/2 + fm.getDescent() + 2;	
				
				int selectorWidth 		= getHeight();
				int verticalLinePosX 	= getWidth()-1 - selectorWidth;
				int arrowHalfSize 		= selectorWidth/4;
				int arrowPosX 			= getWidth()-1 - selectorWidth + selectorWidth/2;
				int arrowPosY 			= getHeight()/2+1 - arrowHalfSize/2;
				
				
				// * Fix Title *
				if(list[comboBox.getSelectedIndex()].compareTo("-") == 0) comboBox.setSelectedIndex(lastHoveringIndex);

				// * Calculate Text Length *
				String title 	= list[comboBox.getSelectedIndex()];
				int strWidth 	= fm.stringWidth(title);	
				int charCount 	= title.length();
				while(strWidth + stringGap > verticalLinePosX) {

					charCount--;
					if(title.length() > charCount)	title = title.substring(0, charCount);
					strWidth = fm.stringWidth(title);					
					if(charCount < 5) break;
				}
				
				// * Draw Title *
				g2d.setFont(getFont());
				g2d.setColor(getForeground());
				g2d.drawString(title, stringGap, textYPos);


				// * Draw vertical Line *
				g2d.setColor(borderColor);
				g2d.drawLine(verticalLinePosX, stroke*4, verticalLinePosX, (int)((getHeight()-stroke*4)));

				// * Draw Arrow *
				g2d.setColor(borderColor);
				g2d.fillPolygon(new int[] {arrowPosX - arrowHalfSize, arrowPosX + arrowHalfSize, arrowPosX}, new int[] {arrowPosY, arrowPosY,arrowPosY + arrowHalfSize}, 3);


				g2d.dispose();
			}
		});

		setRenderer(new ListCellRenderer() {

			@Override
			public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {

				// * Draw List Element *

				JLabel c 		= new JLabel();				
				Border border 	= c.getBorder();
				Border margin 	= new EmptyBorder(2,10,2,0);
				c.setBorder(new CompoundBorder(border, margin));

				c.setOpaque(true);
				c.setForeground(Color.BLACK);
				c.setBackground(getBackground());
				if(isSelected) {

					c.setForeground(Color.WHITE);
					c.setBackground(new Color(180,180,180));

					// * Remember the last Hovering Index, to fix the "-" selecting Index *
					if(((String)value).compareTo("-") != 0) lastHoveringIndex = index;
				}

				// * Draw Seperate Line *				
				if(((String)value).compareTo("-") == 0) {

					JPanel p = new JPanel() {
						@Override
						public void paintComponent(Graphics g) {

							Graphics2D g2d = (Graphics2D) g.create();
							g2d.setColor(new Color(120,120,120,100));
							g2d.fillRect(0, 0, getWidth(), getHeight());
						}							
					};			

					p.setOpaque(true);
					p.setLayout(null) ;		

					int stroke 		= fframe.monitor.convertFloatToRealHeight(0.0015f);					
					p.setPreferredSize(new Dimension(getWidth(),stroke));
					return p;
				}


				// * Draw Text *
				c.setFont(getFont());
				c.setText((String)value);
				return c;
			}
		});

				

		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				try {

					repaint();			
				}
				catch(Exception e) {}
			}});

		

		Toolkit.getDefaultToolkit().addAWTEventListener(new AWTEventListener() {

			@Override
			public void eventDispatched(AWTEvent event) {

				if(fframe.monitor.width != monitorOldWidth || fframe.monitor.height != monitorOldHeight) {

					monitorOldWidth = fframe.monitor.width;
					monitorOldHeight = fframe.monitor.height;
					updateFont();
					updateBounds();
					repaint();			
				}
			}
		}, AWTEvent.PAINT_EVENT_MASK);
	}



	public void setRadius(float r) {
		this.radius = r;
	}

	public void setBorderColor(Color c) {
		this.borderColor = c;
	}
	
	public void setFont(Font font) {				
		super.setFont(font);

		if(fframe == null) return;

		fontName 	= font.getName();
		fontStyle 	= font.getStyle();
		fontSize 	= fframe.monitor.convertRealToFloatHeight(font.getSize());
	}	

	public void setFontSize(float size) {

		this.fontSize = size;
		updateFont();
	}

	public void setFontName(String fontName) {

		this.fontName = fontName;
		updateFont();
	}

	public void setFontStyle(int fontStyle) {

		this.fontStyle = fontStyle;
		updateFont();
	}

	public void setBounds(float x, float y, float width, float height) {

		this.x 		= x;
		this.y 		= y;
		this.width 	= width;
		this.height 	= height;	
		updateBounds();
	}

	public void setLocation(float x, float y) {

		this.x 		= x;
		this.y 		= y;		
		updateBounds();
	}

	public void setSize(float width, float height) {

		this.width 	= width;
		this.height 	= height;	
		updateBounds();
	}

	private void updateBounds() {

		setBounds(fframe.monitor.convertFloatToRealWidth(x), fframe.monitor.convertFloatToRealHeight(y), fframe.monitor.convertFloatToRealWidth(width), fframe.monitor.convertFloatToRealHeight(height));
	}

	private void updateFont() {

		Font font 	= new Font(fontName, fontStyle, fframe.monitor.convertFloatToRealHeight(fontSize));	
		super.setFont(font);	
		repaint();		
	}	

}
