package ffilechooser;

import java.awt.AWTEvent;
import java.awt.GraphicsConfiguration;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.AWTEventListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;

import javax.swing.JFrame;

public class FMonitor {


	public interface MonitorListener {

		public void monitorResolutionChanged();
		public void frameResized();
		public void frameMoved();
	}	

	MonitorListener monitorListener;


	static public final int DOCK_SITE_BOTTOM 	= 0;
	static public final int DOCK_SITE_LEFT 		= 1;
	static public final int DOCK_SITE_RIGHT 		= 2;
	public int dockSite 							= 0;
	public int dockHeight 						= 0;


	// * Multiplicator to get real Squares *
	static float squareMul = 1.0f;  // will be calculated with every Monitor Screen Resolution switch

	public int width;
	public int height;
	public int toolBarHeight = 0;

	private int oldMonitorWidth;
	private int oldMonitorHeight;


	boolean monitorResolutionChangedFlag = false;

	boolean noWaitIfMonitorWasCreated = true;

	JFrame jFrame;

	public FMonitor(JFrame jFrame, final MonitorListener monitorListener) {

		this.jFrame 				= jFrame;
		this.monitorListener 	= monitorListener;


		// * Monitor Resize Listener *

		Toolkit.getDefaultToolkit().addAWTEventListener(new AWTEventListener() {

			@Override
			public void eventDispatched(AWTEvent event) {

				monitorResolutionChangedCheck();				
				if(monitorResolutionChangedFlag) {	

					if(noWaitIfMonitorWasCreated == false) {

						// * Give Repaint time to resize the Frame, and prevent "componentMoved" and "componentResized" Events *
						try {Thread.sleep(1000);	} catch (Exception e) {e.printStackTrace();}

					}
					else noWaitIfMonitorWasCreated = false;

					// * At the end, clear flag *
					monitorResolutionChangedFlag = false;			

					monitorListener.monitorResolutionChanged();
				}				
			}
		}, AWTEvent.PAINT_EVENT_MASK);


		jFrame.addComponentListener(new ComponentAdapter() {

			public void componentMoved(ComponentEvent e) {

				monitorListener.frameMoved();
			}
			public void componentResized(ComponentEvent e) {

				monitorListener.frameResized();
			}
			public void componentShown(ComponentEvent e) {
			}

			public void componentHidden(ComponentEvent e) {
			}
		});



		updateMonitorDatas();
	}


	private void monitorResolutionChangedCheck() {

		if(monitorResolutionChangedFlag) return;

		updateMonitorDatas();	


		// * Resize image only if the Monitor resolution has changed *
		if(oldMonitorWidth != width || height != oldMonitorHeight) {

			oldMonitorWidth 		= width;
			oldMonitorHeight 	= height;			

			monitorResolutionChangedFlag = true;
		}			
	}	

	private void updateMonitorDatas() {

		try {

			GraphicsEnvironment env 			= GraphicsEnvironment.getLocalGraphicsEnvironment();  
			GraphicsDevice[] screens 		= env.getScreenDevices();  

			GraphicsConfiguration config 	= jFrame.getGraphicsConfiguration();
			GraphicsDevice myScreen 			= config.getDevice();

			for (GraphicsDevice screen : screens) {  

				Rectangle screenBounds 	= screen.getDefaultConfiguration().getBounds();
				screenBounds.x 			= 0;  // der 2. Monitor liefert anstatt xPos = 0 eine - 1920, deshalb Positionen auf null setzen  
				screenBounds.y 			= 0;  



				Insets scnMax = Toolkit.getDefaultToolkit().getScreenInsets(screen.getDefaultConfiguration());
				if(scnMax.top > 0) {

					toolBarHeight 	= scnMax.top;
				}
				if(scnMax.bottom > 0) {

					dockHeight 		= scnMax.bottom;
					dockSite 		= DOCK_SITE_BOTTOM;
				}
				if(scnMax.left > 0) {

					dockHeight 		= scnMax.left;
					dockSite 		= DOCK_SITE_LEFT;
				}
				if(scnMax.right > 0) {

					dockHeight 		= scnMax.right;
					dockSite 		= DOCK_SITE_RIGHT;
				}

				// * If this is the Monitor with the App, take the data *
				if (screen.equals(myScreen)) {

					width = screenBounds.width;
					height = screenBounds.height;	

					squareMul 		= (float)width/height;	
				}								
			}			
		}
		catch(Exception e) {e.printStackTrace();}
	}


	public int convertFloatToRealWidth(double v) {			
		return (int) ((float)width/1.0f*v);
	}
	public int convertFloatToRealHeight(double v) {			
		return (int) ((float)height/1.0f*v);
	}

	public float convertRealToFloatWidth(int v) {		
		return (1.0f/(float)width)*v;
	}
	public float convertRealToFloatHeight(int v) {		
		return (1.0f/(float)height)*v;
	}

	// * Special version of "convertFloatToRealWidth() ... we need not "int", we need "float" (Font width: 13.5f) *
	public float convertFloatToRealfWidth(double v) {			
		return (float) ((float)width/1.0f*v);
	}

}
