package ffilechooser;

import java.awt.AWTEvent;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.Toolkit;
import java.awt.event.AWTEventListener;
import java.awt.geom.Ellipse2D;
import java.awt.image.BufferedImage;

import javax.swing.JComponent;
import javax.swing.JFrame;

import ffilechooser.FMonitor.MonitorListener;

public class FRadioButton extends JComponent {

	float x 			= 0;
	float y 			= 0;
	float width 		= 0.02f;
	float height 	= 0.02f;

	boolean selected = false;
	

	float monitorOldWidth = 0;
	float monitorOldHeight = 0;
	
	FFrame fframe;
	


	public FRadioButton(final FFrame fframe) {

		this.fframe = fframe;

		Toolkit.getDefaultToolkit().addAWTEventListener(new AWTEventListener() {

			@Override
			public void eventDispatched(AWTEvent event) {

				if(fframe.monitor.width != monitorOldWidth || fframe.monitor.height != monitorOldHeight) {

					monitorOldWidth = fframe.monitor.width;
					monitorOldHeight = fframe.monitor.height;
					updateButton();
					repaint();			
				}
			}
		}, AWTEvent.PAINT_EVENT_MASK);
	}

	public void setSelected(boolean selected) {
		this.selected = selected;
	}
	
	
	private void updateButton() {
		updateBounds();
		repaint();		
	}

	public void setBounds(float x, float y, float width, float height) {

		this.x 		= x;
		this.y 		= y;
		this.width 	= width;
		this.height 	= height;	
		updateButton();
	}

	public void setLocation(float x, float y) {

		this.x 		= x;
		this.y 		= y;		
		updateButton();
	}

	public void setSize(float width, float height) {

		this.width 	= width;
		this.height 	= height;	
		updateButton();
	}

	private void updateBounds() {

		int w = fframe.monitor.convertFloatToRealWidth(width);
		int h = fframe.monitor.convertFloatToRealHeight(height);

		if(w % 2 == 1) {

			w++;
			width = fframe.monitor.convertRealToFloatWidth(w);
		}
		if(h % 2 == 1) {

			h++;
			height = fframe.monitor.convertRealToFloatHeight(h);
		}		

		setBounds(fframe.monitor.convertFloatToRealWidth(x), fframe.monitor.convertFloatToRealHeight(y), fframe.monitor.convertFloatToRealWidth(width), fframe.monitor.convertFloatToRealHeight(height));
	}


	@Override
	protected void paintComponent(Graphics g) {		
		super.paintComponent(g);

		Graphics2D g2d = (Graphics2D)g;
		g2d.setRenderingHint(RenderingHints.KEY_ALPHA_INTERPOLATION, RenderingHints.VALUE_ALPHA_INTERPOLATION_QUALITY);
		g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		g2d.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);
		g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,   RenderingHints.VALUE_ANTIALIAS_ON);
		g2d.setRenderingHint(RenderingHints.KEY_STROKE_CONTROL, RenderingHints.VALUE_STROKE_PURE);
		g2d.setStroke(new BasicStroke(1.0f));


		float w1 = fframe.monitor.convertFloatToRealWidth(width);
		float h1 = fframe.monitor.convertFloatToRealHeight(height);
		float w2 = (int) (w1*0.7f);
		float h2 = (int) (h1*0.7f);

		Ellipse2D ellipse1 = new Ellipse2D.Float();
		Ellipse2D ellipse2 = new Ellipse2D.Float();
		
		ellipse1.setFrame(1,1, w1 - 2 * 1, h1 - 2 * 1);
		ellipse2.setFrame(1 + getWidth()/2 - w2/2, 1 + getHeight()/2 - h2/2, w2 - 2 * 1, h2 - 2 * 1);
        g2d.draw(ellipse1);
        if(selected) g2d.fill(ellipse2);


		g2d.dispose();		
	}

}
