package ffilechooser;
import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.GraphicsConfiguration;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.RenderingHints;
import java.awt.Transparency;
import java.awt.image.BufferedImage;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.RandomAccessFile;
import java.nio.channels.FileChannel;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Vector;

import javax.swing.DefaultRowSorter;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JTable;
import javax.swing.RowSorter;
import javax.swing.RowSorter.SortKey;
import javax.swing.table.DefaultTableModel;
import javax.swing.SortOrder;




public class Functions {

	static public File[] getDir(String path, final boolean showHiddenFiles) {

		File dir = new File(path);
		File[] dirList = dir.listFiles(new FileFilter() {

			@Override
			public boolean accept(File file) {

				if(file.getName().compareTo("dev") == 0 || file.getName().toLowerCase().compareTo(".ds_store") == 0 ||(!showHiddenFiles && file.getName().startsWith("."))) return false;				
				return true;
			}	
		});

		return dirList;
	}			


	public static void rmDir(File file) {

		if (!file.exists()) return;
		if (file.isDirectory()) {
			for (File f : file.listFiles()) {

				try {rmDir(f); } catch(Exception e) { e.printStackTrace();}
			}
			file.delete();
		} else {
			file.delete();
		}		
	} 


	public static long getFolderSize(File directory) {

		long length = 0;
		for (File file : directory.listFiles()) {

			if (file.isFile()) length += file.length();
			else {

				if(isLinkFile(file) == null) 	length += getFolderSize(file);
			}
		}
		return length;
	}



	public static ImageIcon iconToImageIcon(Icon icon) {

		int w = icon.getIconWidth();
		int h = icon.getIconHeight();

		BufferedImage img = new BufferedImage(w, h, BufferedImage.TYPE_INT_ARGB);
		Graphics2D g2 = img.createGraphics();
		icon.paintIcon(null, g2, 0, 0);
		g2.dispose();

		return new ImageIcon(img);
	}


	public static BufferedImage iconToBufferedImage(Icon icon) {

		int w = icon.getIconWidth();
		int h = icon.getIconHeight();

		float alpha = 1.0f;

		BufferedImage img = new BufferedImage(w, h, BufferedImage.TYPE_INT_ARGB);
		Graphics2D g2 = img.createGraphics();
		g2.setComposite(AlphaComposite.Src);
		icon.paintIcon(null, g2, 0, 0);
		g2.dispose();

		return img;
	}




	static public String convertLengthToMemory(long bytes) {

		if(bytes < 1000) return "" + bytes + " bytes";
		else {

			double k = bytes/1000;    		
			if(k > 1000) {

				double k2 = (int)k/1000; 
				if(k2 > 1000) {

					double k3 = (int)k2/1000;     
					if(k3 > 1000) {

						return "" + String.format("%.1f", (double)((double)bytes/1000/1000/1000/1000)) + " TB";          			
					}
					else return "" + String.format("%.1f", (double)((double)bytes/1000/1000/1000)) + " GB";

				}
				else return "" + String.format("%.1f", (double)((double)bytes/1000/1000)) + " MB";
			}
			else return "" + (int)k + " KB";
		}
	}

	static public String convertToFlatBytes(String s) {

		s = s.toLowerCase();

		s = s.replace("zero bytes", "");
		s = s.replace("bytes", "");
		//s = s.replace("--", "");
		s = s.trim();


		if(s.contains("kb")) {

			s = s.replace(" kb", "");
			float a = Float.parseFloat(s);
			a *= 1000;				
			s = "" + (long)a;
		}
		else if(s.contains("mb")) {

			s = s.replace(" mb", "");
			float a = Float.parseFloat(s);
			a *= 1000000;				
			s = "" + (long)a;
		}
		else if(s.contains("gb")) {

			s = s.replace(" gb", "");
			float a = Float.parseFloat(s);
			a *= 1000000000;				
			s = "" + (long)a;
		}

		int addZeros = 20-s.length();
		for(int i=0;i<addZeros;i++) s = "0"+s;


		return s;
	}





	static byte[] bookmarkHeader = new byte[] {0x62,0x6F,0x6F,0x6B,0x00,0x00,0x00,0x00,0x6D,0x61,0x72,0x6B,0x00,0x00,0x00,0x00};
	static byte[] fileLinkHeader = new byte[] {0x04,0x00,0x00,0x00,0x03,0x03,0x00,0x00,0x00,0x04,0x00,0x00};
	static byte[] nextPartBytes = new byte[] {0x01,0x01,0x00,0x00};
	static byte[] endLinkBytes = new byte[] {0x01,0x06,0x00,0x00};


	synchronized public static Path isLinkFile(File file) {

		Path linkPath = null; 

		if(file == null || file.isDirectory() || file.length() == 0) return linkPath;

		try {

			if(Files.isSymbolicLink(file.toPath())) linkPath = Files.readSymbolicLink(file.toPath());
		} 
		catch (Exception e) {
			e.printStackTrace();
		}		


		if(linkPath == null) {

			// * Wenn kein "isSymbolicLink", dann nachsehen ob "bookmark" link file *

			RandomAccessFile rf;
			try {

				rf = new RandomAccessFile(file, "r");
				rf.seek(0);

				byte[] buffer = new byte[16];
				rf.read(buffer);

				if(Arrays.equals(buffer,bookmarkHeader)) {

					// * Link aus File lesen... *

					String link = "/";

					long pos = 0;
					pos = findFileLinkHeader(rf,file.length());
					if(pos != -1) {

						while(true) {

							String part = readPart(rf);

							if(part != null) 	link += part;							
							else 				break;
						}						
					}


					linkPath = new File(link).toPath();

					//System.out.println("-----> linkPath ("+file.getAbsolutePath()+"): " + linkPath);
				}

				rf.close();

			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}


		// * Wenn kein Symbolic Link, dann NULL, ansnonsten den Link-Path zurueckgeben! *
		return linkPath;
	}


	private static int findFileLinkHeader(RandomAccessFile rf, long fileLength) {

		int pos = 0;

		try {

			byte[] buffer = new byte[12];
			rf.read(buffer);

			while(true) {

				rf.seek(pos);
				rf.read(buffer);
				if(Arrays.equals(buffer,fileLinkHeader)) return pos;

				pos++;

				if(pos + fileLinkHeader.length > fileLength || pos > 200) break;
			}
		}
		catch(Exception e) { e.printStackTrace(); }

		return -1;
	}

	private static String readPart(RandomAccessFile rf) {

		byte[] firstBytes = new byte[4];
		byte[] lastBytes = new byte[4];

		try {

			long pos = rf.getFilePointer();

			// * naechstes Byte lesen, solange != 0 *
			long b = 0;
			do {

				b = rf.read();				
			}
			while(b == 0);

			pos = rf.getFilePointer();

			// * 1 byte zurueck, weil das in den Buffer rein muss *
			rf.seek(pos-1);

			// * 2 x 4 bytes lesen *
			rf.read(firstBytes);
			rf.read(lastBytes);

			//System.out.println("firstBytes: " + Functions.byteArrayToHexString(firstBytes));
			//System.out.println("lastBytes: " + Functions.byteArrayToHexString(lastBytes));

			// * Wenn Parts Ende Bytes, dann Ende *
			if(Arrays.equals(lastBytes,endLinkBytes)) 	return null;

			// * Wenn Next Part Bytes, dann weiter einlesen *
			if(Arrays.equals(lastBytes,nextPartBytes)) {

				int pathPartLen = firstBytes[0];
				byte[] buffer = new byte[pathPartLen];
				rf.read(buffer);								
				return new String(buffer) + "/";
			}
		}
		catch(Exception e) {e.printStackTrace();}

		return null;
	}





	static final String HEXES = "0123456789ABCDEF";

	public static String byteToHexString( byte by ) {

		byte [] raw = {by};

		if ( raw == null ) {
			return null;
		}
		final StringBuilder hex = new StringBuilder( 2 * raw.length );
		for ( final byte b : raw ) {
			hex.append(HEXES.charAt((b & 0xF0) >> 4)).append(HEXES.charAt((b & 0x0F)));
			hex.append(":");
		}

		String hexString = hex.toString().substring(0, hex.length()-1);

		return hexString;
	}

	public static String byteArrayToHexString( byte [] raw ) {
		if ( raw == null ) {
			return null;
		}
		final StringBuilder hex = new StringBuilder( 2 * raw.length );
		for ( final byte b : raw ) {
			hex.append(HEXES.charAt((b & 0xF0) >> 4)).append(HEXES.charAt((b & 0x0F)));
			hex.append(":");
		}

		String hexString = hex.toString().substring(0, hex.length()-1);

		return hexString;
	}

	public static byte[] hexStringToByteArray(String s) {

		int len = s.length();

		byte[] data = new byte[len / 2];
		for (int i = 0; i < len; i += 2) {
			data[i / 2] = (byte) ((Character.digit(s.charAt(i), 16) << 4)  + Character.digit(s.charAt(i+1), 16));
		}
		return data;
	}


	public static int hexStringToInt(String s) {

		int len = s.length();

		byte[] data = new byte[len / 2];
		for (int i = 0; i < len; i += 2) {
			data[i / 2] = (byte) ((Character.digit(s.charAt(i), 16) << 4)  + Character.digit(s.charAt(i+1), 16));
		}

		int summe=0;
		for(int i=0;i<data.length;i++) {

			summe += data[i];
		}

		return summe;
	}


}
